@@ -11,3 +11,5 @@
 #undef HAVE_STRUCT_LINGER
 #undef HAVE_NSL
 #undef GNOME_VFS_DEFAULT_BACKEND
+#undef USING_OAF
+#undef HAVE_GCONF
