@@ -21,13 +21,10 @@
 
    Author: Ettore Perazzoli <ettore@comm2000.it> */
 
-/* FIXME: `operation_in_progress' should be set immediately.  */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <libgnorba/gnorba.h>
 #include <orb/orbit.h>
 
 #include "gnome-vfs.h"
@@ -42,7 +39,7 @@
 #define RETURN_IF_SLAVE_BUSY(slave)					  \
 	g_return_val_if_fail						  \
 		(slave->operation_in_progress == GNOME_VFS_ASYNC_OP_NONE, \
-		 GNOME_VFS_ERROR_INPROGRESS)
+		 GNOME_VFS_ERROR_IN_PROGRESS)
 
 
 GnomeVFSResult	 
@@ -61,9 +58,9 @@ corba_gnome_vfs_async_open (GnomeVFSAsyncHandle **handle_return,
 {
 	GnomeVFSSlaveProcess *slave;
 
-	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	slave = gnome_vfs_slave_process_new ();
 	if (slave == NULL)
@@ -106,13 +103,13 @@ corba_gnome_vfs_async_open_as_channel (GnomeVFSAsyncHandle **handle_return,
 {
 	GnomeVFSSlaveProcess *slave;
 
-	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 	g_return_val_if_fail ((open_mode & (GNOME_VFS_OPEN_READ
 					    | GNOME_VFS_OPEN_WRITE))
 			      != (GNOME_VFS_OPEN_READ | GNOME_VFS_OPEN_WRITE),
-			      GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BADPARAMS);
+			      GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	slave = gnome_vfs_slave_process_new ();
 	if (slave == NULL)
@@ -159,11 +156,11 @@ corba_gnome_vfs_async_create (GnomeVFSAsyncHandle **handle_return,
 {
 	GnomeVFSSlaveProcess *slave;
 
-	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail ((open_mode & GNOME_VFS_OPEN_WRITE), GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (! (open_mode & GNOME_VFS_OPEN_READ), GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail ((open_mode & GNOME_VFS_OPEN_WRITE), GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (! (open_mode & GNOME_VFS_OPEN_READ), GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	slave = gnome_vfs_slave_process_new ();
 	if (slave == NULL)
@@ -210,13 +207,13 @@ corba_gnome_vfs_async_create_as_channel (GnomeVFSAsyncHandle **handle_return,
 {
 	GnomeVFSSlaveProcess *slave;
 
-	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail ((open_mode & GNOME_VFS_OPEN_WRITE), GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail ((open_mode & GNOME_VFS_OPEN_WRITE), GNOME_VFS_ERROR_BAD_PARAMETERS);
 	g_return_val_if_fail ((open_mode & GNOME_VFS_OPEN_READ)
 			      && (open_mode & GNOME_VFS_OPEN_WRITE),
-			      GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BADPARAMS);
+			      GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	slave = gnome_vfs_slave_process_new ();
 	if (slave == NULL)
@@ -255,8 +252,8 @@ corba_gnome_vfs_async_close (GnomeVFSAsyncHandle *handle,
 {
 	GnomeVFSSlaveProcess *slave;
 
-	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	slave = (GnomeVFSSlaveProcess *) handle;
 
@@ -293,9 +290,9 @@ corba_gnome_vfs_async_read (GnomeVFSAsyncHandle *handle,
 {
 	GnomeVFSSlaveProcess *slave;
 
-	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (buffer != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (buffer != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	slave = (GnomeVFSSlaveProcess *) handle;
 
@@ -334,9 +331,9 @@ corba_gnome_vfs_async_write (GnomeVFSAsyncHandle *handle,
 	GnomeVFSSlaveProcess *slave;
 	GNOME_VFS_Buffer *corba_buffer;
 
-	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (buffer != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (buffer != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	slave = (GnomeVFSSlaveProcess *) handle;
 
@@ -375,29 +372,55 @@ G_STMT_START{								\
 			;						\
 }G_STMT_END
 
+static GNOME_VFS_Slave_URIList *
+gnome_vfs_uri_list_to_corba_uri_list (GList *vfs_uri_list)
+{
+	GNOME_VFS_Slave_URIList *corba_list;
+	GList *p;
+	int length, i;
+	char *string;
+
+	length = g_list_length (vfs_uri_list);
+
+	corba_list = GNOME_VFS_Slave_URIList__alloc ();
+	corba_list->_maximum = length;
+	corba_list->_length = length;
+	corba_list->_buffer = CORBA_sequence_CORBA_string_allocbuf (length);
+
+	for (i = 0, p = vfs_uri_list; i < length; i++, p = p->next) {
+		string = gnome_vfs_uri_to_string (p->data, GNOME_VFS_URI_HIDE_NONE);
+		corba_list->_buffer[i] = CORBA_string_dup (string);
+		g_free (string);
+	}
+
+	CORBA_sequence_set_release (corba_list, TRUE);
+
+	return corba_list;
+}
+
 GnomeVFSResult
 corba_gnome_vfs_async_get_file_info (GnomeVFSAsyncHandle **handle_return,
-				     const char *uri,
+				     GList *uris,
 				     GnomeVFSFileInfoOptions options,
-				     const char *meta_keys[],
+				     const char * const meta_keys[],
 				     GnomeVFSAsyncGetFileInfoCallback callback,
 				     gpointer callback_data);
 GnomeVFSResult
 corba_gnome_vfs_async_get_file_info (GnomeVFSAsyncHandle **handle_return,
-				     const char *uri,
+				     GList *uris,
 				     GnomeVFSFileInfoOptions options,
-				     const char *meta_keys[],
+				     const char * const meta_keys[],
 				     GnomeVFSAsyncGetFileInfoCallback callback,
 				     gpointer callback_data)
 {
 	GnomeVFSSlaveProcess *slave;
+	GNOME_VFS_Slave_URIList *uri_list;
 	GNOME_VFS_MetadataKeyList my_meta_keys;
 	int num_meta_keys;
 	GnomeVFSAsyncDirectoryOpInfo *op_info; /* piggyback */
 
-	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	slave = gnome_vfs_slave_process_new();
 	if (slave == NULL)
@@ -419,17 +442,22 @@ corba_gnome_vfs_async_get_file_info (GnomeVFSAsyncHandle **handle_return,
 	slave->callback = callback;
 	slave->callback_data = callback_data;
 
+	uri_list = gnome_vfs_uri_list_to_corba_uri_list (uris);
+
 	my_meta_keys._length = my_meta_keys._maximum = num_meta_keys;
-	my_meta_keys._buffer = (char **)meta_keys;
+	my_meta_keys._buffer = (char **) meta_keys;
 	CORBA_sequence_set_release (&my_meta_keys, FALSE);
 
 	slave->operation_in_progress = GNOME_VFS_ASYNC_OP_GET_FILE_INFO;
 
 	GNOME_VFS_Slave_Request_get_file_info (slave->request_objref,
-					       uri,
+					       uri_list,
 					       options,
 					       &my_meta_keys,
 					       &slave->ev);
+
+	CORBA_free (uri_list);
+
 	if (slave->ev._major != CORBA_NO_EXCEPTION) {
 		gnome_vfs_slave_process_destroy (slave);
 		return GNOME_VFS_ERROR_INTERNAL;
@@ -444,7 +472,7 @@ GnomeVFSResult
 corba_gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 				      const gchar *uri,
 				      GnomeVFSFileInfoOptions options,
-				      const gchar *meta_keys[],
+				      const gchar * const meta_keys[],
 				      GnomeVFSDirectorySortRule sort_rules[],
 				      gboolean reverse_order,
 				      GnomeVFSDirectoryFilterType filter_type,
@@ -457,7 +485,7 @@ GnomeVFSResult
 corba_gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 				      const gchar *uri,
 				      GnomeVFSFileInfoOptions options,
-				      const gchar *meta_keys[],
+				      const gchar * const meta_keys[],
 				      GnomeVFSDirectorySortRule sort_rules[],
 				      gboolean reverse_order,
 				      GnomeVFSDirectoryFilterType filter_type,
@@ -475,9 +503,9 @@ corba_gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 	guint i;
 	GnomeVFSAsyncDirectoryOpInfo *op_info;
 
-	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (callback != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	slave = gnome_vfs_slave_process_new ();
 	if (slave == NULL)
@@ -549,7 +577,7 @@ GnomeVFSResult
 corba_gnome_vfs_async_load_directory_uri (GnomeVFSAsyncHandle **handle_return,
 					  GnomeVFSURI *uri,
 					  GnomeVFSFileInfoOptions options,
-					  const gchar *meta_keys[],
+					  const gchar * const meta_keys[],
 					  GnomeVFSDirectorySortRule sort_rules[],
 					  gboolean reverse_order,
 					  GnomeVFSDirectoryFilterType filter_type,
@@ -562,7 +590,7 @@ GnomeVFSResult
 corba_gnome_vfs_async_load_directory_uri (GnomeVFSAsyncHandle **handle_return,
 					  GnomeVFSURI *uri,
 					  GnomeVFSFileInfoOptions options,
-					  const gchar *meta_keys[],
+					  const gchar * const meta_keys[],
 					  GnomeVFSDirectorySortRule sort_rules[],
 					  gboolean reverse_order,
 					  GnomeVFSDirectoryFilterType filter_type,
@@ -590,33 +618,6 @@ corba_gnome_vfs_async_load_directory_uri (GnomeVFSAsyncHandle **handle_return,
 	return retval;
 }
 
-#if 0
-static GNOME_VFS_Slave_FileNameList *
-g_list_to_file_name_list (const GList *list)
-{
-	GNOME_VFS_Slave_FileNameList *new;
-	const GList *p;
-	guint length, i;
-
-	length = g_list_length ((GList *) list); /* FIXME dammit glib */
-
-	/* FIXME it is a waste to re-alloc all the strings, but I am not sure
-           how to free the sequence if I don't do so otherwise. */
-
-	new = GNOME_VFS_Slave_FileNameList__alloc ();
-	new->_maximum = length;
-	new->_length = length;
-	new->_buffer = CORBA_sequence_CORBA_string_allocbuf (length);
-
-	for (i = 0, p = list; i < length; i++, p = p->next)
-		new->_buffer[i] = CORBA_string_dup (p->data);
-
-	CORBA_sequence_set_release (new, TRUE);
-
-	return new;
-}
-#endif
-
 GnomeVFSResult
 corba_gnome_vfs_async_xfer (GnomeVFSAsyncHandle **handle_return,
 			    const gchar *source_dir,
@@ -626,8 +627,10 @@ corba_gnome_vfs_async_xfer (GnomeVFSAsyncHandle **handle_return,
 			    GnomeVFSXferOptions xfer_options,
 			    GnomeVFSXferErrorMode error_mode,
 			    GnomeVFSXferOverwriteMode overwrite_mode,
-			    GnomeVFSAsyncXferProgressCallback progress_callback,
-			    gpointer data);
+			    GnomeVFSAsyncXferProgressCallback progress_update_callback,
+			    gpointer update_callback_data,
+			    GnomeVFSXferProgressCallback progress_sync_callback,
+			    gpointer sync_callback_data);
 GnomeVFSResult
 corba_gnome_vfs_async_xfer (GnomeVFSAsyncHandle **handle_return,
 			    const gchar *source_dir,
@@ -637,9 +640,16 @@ corba_gnome_vfs_async_xfer (GnomeVFSAsyncHandle **handle_return,
 			    GnomeVFSXferOptions xfer_options,
 			    GnomeVFSXferErrorMode error_mode,
 			    GnomeVFSXferOverwriteMode overwrite_mode,
-			    GnomeVFSAsyncXferProgressCallback progress_callback,
-			    gpointer data)
+			    GnomeVFSAsyncXferProgressCallback progress_update_callback,
+			    gpointer update_callback_data,
+			    GnomeVFSXferProgressCallback progress_sync_callback,
+			    gpointer sync_callback_data)
 {
+	/* FIXME bugzilla.eazel.com 1122:
+	 * 
+	 * Update to pass progress_sync_callback properly.
+	 * 
+	 */
 	GNOME_VFS_Slave_FileNameList corba_source_list;
 	GNOME_VFS_Slave_FileNameList corba_target_list;
 	GnomeVFSXferProgressInfo *progress_info;
@@ -648,11 +658,8 @@ corba_gnome_vfs_async_xfer (GnomeVFSAsyncHandle **handle_return,
 	int i;
 	const GList *cur;
 
-	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (source_dir != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (source_name_list != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (target_dir != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (progress_callback != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle_return != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (progress_update_callback != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	slave = gnome_vfs_slave_process_new ();
 	if (slave == NULL)
@@ -681,8 +688,8 @@ corba_gnome_vfs_async_xfer (GnomeVFSAsyncHandle **handle_return,
 	}
 
 	slave->operation_in_progress = GNOME_VFS_ASYNC_OP_XFER;
-	slave->callback = progress_callback;
-	slave->callback_data = data;
+	slave->callback = progress_update_callback;
+	slave->callback_data = update_callback_data;
 
 	op_info = &slave->op_info.xfer;
 	op_info->xfer_options = xfer_options;
@@ -715,7 +722,7 @@ corba_gnome_vfs_async_cancel (GnomeVFSAsyncHandle *handle)
 {
 	GnomeVFSSlaveProcess *slave;
 
-	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	slave = (GnomeVFSSlaveProcess *) handle;
 
@@ -728,7 +735,7 @@ corba_gnome_vfs_async_cancel (GnomeVFSAsyncHandle *handle)
 	
 	/* return GNOME_VFS_OK; */
 
-	return GNOME_VFS_ERROR_NOTSUPPORTED;
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
 guint
