@@ -31,7 +31,6 @@
 #include <sys/socket.h>
 #include <unistd.h>
      
-#include <libgnorba/gnorba.h>
 #include <orb/orbit.h>
 
 #include "gnome-vfs.h"
@@ -85,7 +84,7 @@ static void
 impl_Notify_reset (PortableServer_Servant servant,
 		   CORBA_Environment *ev)
 {
-	/* FIXME */
+	/* FIXME bugzilla.eazel.com 1124: */
 #if 0
 	GnomeVFSSlaveProcess *slave;
 	GnomeVFSSlaveProcessResetCallback callback;
@@ -199,7 +198,8 @@ impl_Notify_open_as_channel (PortableServer_Servant servant,
 			saddr.sun_family = AF_UNIX;
 			strncpy (saddr.sun_path, sock_path,
 				 sizeof (saddr.sun_path));
-			r = connect (fd, &saddr, SUN_LEN (&saddr));
+			r = connect (fd, (struct sockaddr *)&saddr,
+				     SUN_LEN (&saddr));
 			if (r < 0) {
 				g_warning (_("Cannot connect socket `%s': %s"),
 					   saddr.sun_path, g_strerror (errno));
@@ -369,7 +369,7 @@ impl_Notify_xfer_file_start (PortableServer_Servant servant,
 	g_free (progress_info->source_name);
 	g_free (progress_info->target_name);
 
-	progress_info->phase = GNOME_VFS_XFER_PHASE_XFERRING;
+	progress_info->phase = GNOME_VFS_XFER_PHASE_COPYING;
 	progress_info->file_index++;
 	progress_info->source_name = g_strdup (source_uri);
 	progress_info->target_name = g_strdup (target_uri);
@@ -402,7 +402,7 @@ impl_Notify_xfer_file_progress (PortableServer_Servant servant,
 	progress_info = &op_info->progress_info;
 	callback = (GnomeVFSAsyncXferProgressCallback) slave->callback;
 
-	progress_info->phase = GNOME_VFS_XFER_PHASE_XFERRING;
+	progress_info->phase = GNOME_VFS_XFER_PHASE_COPYING;
 	progress_info->bytes_copied = bytes_copied;
 	progress_info->total_bytes_copied = total_bytes_copied;
 
@@ -676,18 +676,24 @@ impl_Notify_load_directory (PortableServer_Servant servant,
 	callback = (GnomeVFSAsyncDirectoryLoadCallback) slave->callback;
 	(* callback) ((GnomeVFSAsyncHandle *) slave, result,
 		      list, files->_length, slave->callback_data);
+}
 
-	/* FIXME kill process? */
+static gchar *
+strdup_or_null (const gchar *string)
+{
+	return string[0] == '\0' ? NULL : g_strdup (string);
 }
 
 static void
 impl_Notify_get_file_info (PortableServer_Servant servant,
-			   const GNOME_VFS_Result result,
-			   const GNOME_VFS_Slave_FileInfo *file_info,
+			   const GNOME_VFS_Slave_GetFileInfoResultList *results,
 			   CORBA_Environment *ev)
 {
 	GnomeVFSSlaveProcess *slave;
+	GList *result_list, *p;
+	GnomeVFSGetFileInfoResult *result_item;
 	GnomeVFSAsyncGetFileInfoCallback callback;
+	const GNOME_VFS_Slave_FileInfo *file_info;
 	GnomeVFSFileInfo *info;
 	char **meta_keys;
 	int num_meta_keys;
@@ -703,42 +709,54 @@ impl_Notify_get_file_info (PortableServer_Servant servant,
 	meta_keys = slave->op_info.directory.meta_keys;
 	num_meta_keys = slave->op_info.directory.num_meta_keys;
 
-	info = gnome_vfs_file_info_new ();
+	result_list = NULL;
+	for (i = 0; i < results->_length; i++) {
+		file_info = &results->_buffer[i].file_info;
 
-	memcpy (info, (GnomeVFSFileInfo *) file_info->data._buffer,
-		file_info->data._length);
-	info->refcount = 1; /* stupid memcpy, doh doh doh */
+		/* Create a file info object from the flat one on the wire. */
+		info = gnome_vfs_file_info_new ();
+		g_assert (file_info->data._length == sizeof (GnomeVFSFileInfo));
+		*info = * (GnomeVFSFileInfo *) file_info->data._buffer;
+		info->refcount = 1; /* stupid memcpy, doh doh doh */
 
-	if (file_info->name[0] == 0)
-		info->name = NULL;
-	else
-		info->name = g_strdup (file_info->name);
+		info->name = strdup_or_null (file_info->name);
+		info->mime_type = strdup_or_null (file_info->mime_type);
+		info->symlink_name = strdup_or_null (file_info->symlink_name);
+		
+		info->metadata_list = NULL;
+		set_metadata_from_response (info,
+					    (GNOME_VFS_Slave_MetadataResponseList *)&file_info->metadata_response,
+					    (const char **)meta_keys);
 
-	if (file_info->mime_type[0] == 0)
-		info->mime_type = NULL;
-	else
-		info->mime_type = g_strdup (file_info->mime_type);
+		result_item = g_new (GnomeVFSGetFileInfoResult, 1);
 
-	if (file_info->symlink_name[0] == 0)
-		info->symlink_name = NULL;
-	else
-		info->symlink_name
-			= g_strdup (file_info->symlink_name);
+		result_item->uri = gnome_vfs_uri_new (results->_buffer[i].uri);
+		result_item->result = results->_buffer[i].result;
+		result_item->file_info = info;
 
-	info->metadata_list = NULL;
-	set_metadata_from_response (info,
-				    (GNOME_VFS_Slave_MetadataResponseList *)&file_info->metadata_response,
-				    (const char **)meta_keys);
+		result_list = g_list_prepend (result_list, result_item);
+	}
+	result_list = g_list_reverse (result_list);
 
 	slave->operation_in_progress = GNOME_VFS_ASYNC_OP_NONE;
 
 	callback = (GnomeVFSAsyncGetFileInfoCallback)slave->callback;
-	(* callback) ((GnomeVFSAsyncHandle *) slave, result, info, slave->callback_data);
+	(* callback) ((GnomeVFSAsyncHandle *) slave,
+		      result_list,
+		      slave->callback_data);
+
+	for (p = result_list; p != NULL; p = p->next) {
+		result_item = p->data;
+
+		gnome_vfs_uri_unref (result_item->uri);
+		gnome_vfs_file_info_unref (result_item->file_info);
+		g_free (result_item);
+	}
+	g_list_free (result_list);
 
 	for (i = 0; i < num_meta_keys; i++)
 		g_free (meta_keys[i]);
 	g_free (meta_keys);
-	gnome_vfs_file_info_unref(info);
 }
 
 
