@@ -52,6 +52,8 @@
 
 #include "gnome-vfs-slave.h"
 
+#include <liboaf/liboaf.h>
+
 
 /*  #define SLAVE_DEBUG */
 
@@ -497,7 +499,7 @@ serve_channel_read (GnomeVFSHandle *handle,
 			continue;
 
 		if (result != GNOME_VFS_OK) {
-			/* FIXME */
+			/* FIXME bugzilla.eazel.com 1226: */
 			g_warning (_("Error reading: %s"),
 				   gnome_vfs_result_to_string (gnome_vfs_result_from_errno ()));
 			return;
@@ -519,7 +521,7 @@ serve_channel_read (GnomeVFSHandle *handle,
 				if (errno == EINTR)
 					continue;
 
-				/* FIXME */
+				/* FIXME bugzilla.eazel.com 1226: */
 				g_warning (_("Error writing: %s"),
 					   g_strerror (errno));
 				return;
@@ -568,7 +570,7 @@ serve_channel_write (GnomeVFSHandle *handle,
 				continue;
 
 			if (result != GNOME_VFS_OK) {
-				/* FIXME */
+				/* FIXME bugzilla.eazel.com 1226: */
 				g_warning (_("Cannot write: %s"),
 					   g_strerror (errno));
 				return;
@@ -810,7 +812,8 @@ copy_metadata (GNOME_VFS_Slave_FileInfo *dest,
 {
 	guint i;
 
-	/* FIXME: This is a bit inefficient (because every
+	/* FIXME bugzilla.eazel.com 1120: 
+	   This is a bit inefficient (because every
 	   `gnome_vfs_file_info_get_metadata()' is actually a linear search
 	   with the current implementation), but we are happy with it for now.
 	   In most cases, the number of metadata keys will be very limited.  */
@@ -914,7 +917,7 @@ load_directory_not_sorted (const gchar *uri,
 	if (stopped)
 		GNOME_VFS_Slave_Notify_stop (notify_objref, ev);
 
-	gnome_vfs_file_info_destroy (info);
+	gnome_vfs_file_info_unref (info);
 }
 
 static void
@@ -995,7 +998,7 @@ load_directory_sorted (const gchar *uri,
 
 static void
 impl_Request_get_file_info (PortableServer_Servant servant,
-			    const CORBA_char *uri,
+			    const GNOME_VFS_Slave_URIList *uris,
 			    const GNOME_VFS_Slave_FileInfoOptions info_options,
 			    const GNOME_VFS_MetadataKeyList *meta_keys,
 			    CORBA_Environment *ev)
@@ -1005,8 +1008,9 @@ impl_Request_get_file_info (PortableServer_Servant servant,
 	guint num_meta_keys;
 	int i, j;
 	GnomeVFSFileInfo *info;
-	GNOME_VFS_Slave_FileInfo out_info, *p;
+	GNOME_VFS_Slave_FileInfo *p;
 	guint info_size;
+	GNOME_VFS_Slave_GetFileInfoResultList *result_list;
 
 	num_meta_keys = meta_keys->_length;
 	if (num_meta_keys == 0) {
@@ -1020,49 +1024,61 @@ impl_Request_get_file_info (PortableServer_Servant servant,
 		meta_key_array[i] = NULL;
 	}
 
-	info = gnome_vfs_file_info_new();
-	result = gnome_vfs_get_file_info(uri, info, info_options, (const char **)meta_key_array);
+	result_list = GNOME_VFS_Slave_GetFileInfoResultList__alloc ();
+	result_list->_length = uris->_length;
+	result_list->_maximum = uris->_length;
+	result_list->_buffer = CORBA_sequence_GNOME_VFS_Slave_GetFileInfoResult_allocbuf (uris->_length);
+	CORBA_sequence_set_release (result_list, TRUE);
 
-	p = &out_info;
-	info_size = sizeof (GnomeVFSFileInfo);
-	p->data._length = info_size;
-	p->data._maximum = info_size;
-	p->data._buffer = alloca(info_size);
-	CORBA_sequence_set_release (&p->data, FALSE);
+	for (i = 0; i < uris->_length; i++) {
+		info = gnome_vfs_file_info_new ();
+		result = gnome_vfs_get_file_info (uris->_buffer[i], info, info_options,
+						  (const char **)meta_key_array);
+		
+		result_list->_buffer[i].uri = CORBA_string_dup (uris->_buffer[i]);
+		result_list->_buffer[i].result = result;
+		p = &result_list->_buffer[i].file_info;
 
-	p->name = info->name?info->name:"";
-	p->symlink_name = info->symlink_name?info->symlink_name:"";
-	p->mime_type = info->mime_type?info->mime_type:"";
-
-	p->metadata_response._length = num_meta_keys;
-	p->metadata_response._maximum = num_meta_keys;
-	p->metadata_response._buffer
-		= alloca(sizeof(GNOME_VFS_Slave_MetadataResponse) * num_meta_keys);
-	CORBA_sequence_set_release (&p->metadata_response, FALSE);
-
-	for (j = 0; j < num_meta_keys; j++) {
-		GNOME_VFS_Slave_MetadataResponse *mp;
-		gconstpointer value;
-		guint value_size;
-
-		mp = p->metadata_response._buffer + j;
-
-		mp->found = gnome_vfs_file_info_get_metadata(info, meta_key_array[j], &value, &value_size);
-		if(mp->found) {
-			mp->value._length = value_size;
-			mp->value._buffer = (gpointer)value;
-		} else {
-			mp->value._length = 0;
-			mp->value._maximum = 0;
-			mp->value._buffer = NULL;
+		info_size = sizeof (GnomeVFSFileInfo);
+		p->data._length = info_size;
+		p->data._maximum = info_size;
+		p->data._buffer = alloca (info_size);
+		CORBA_sequence_set_release (&p->data, FALSE);
+		
+		p->name = info->name?info->name:"";
+		p->symlink_name = info->symlink_name?info->symlink_name:"";
+		p->mime_type = info->mime_type?info->mime_type:"";
+		
+		p->metadata_response._length = num_meta_keys;
+		p->metadata_response._maximum = num_meta_keys;
+		p->metadata_response._buffer
+			= alloca(sizeof(GNOME_VFS_Slave_MetadataResponse) * num_meta_keys);
+		CORBA_sequence_set_release (&p->metadata_response, FALSE);
+		
+		for (j = 0; j < num_meta_keys; j++) {
+			GNOME_VFS_Slave_MetadataResponse *mp;
+			gconstpointer value;
+			guint value_size;
+			
+			mp = p->metadata_response._buffer + j;
+			
+			mp->found = gnome_vfs_file_info_get_metadata(info, meta_key_array[j], &value, &value_size);
+			if (mp->found) {
+				mp->value._length = value_size;
+				mp->value._buffer = (gpointer)value;
+			} else {
+				mp->value._length = 0;
+				mp->value._maximum = 0;
+				mp->value._buffer = NULL;
+			}
+			
+			CORBA_sequence_set_release (&mp->value, FALSE);
 		}
-
-		CORBA_sequence_set_release (&mp->value, FALSE);
 	}
 
-	GNOME_VFS_Slave_Notify_get_file_info (notify_objref, (GNOME_VFS_Result)result, &out_info, ev);
+	GNOME_VFS_Slave_Notify_get_file_info (notify_objref, result_list, ev);
 
-	gnome_vfs_file_info_unref(info);
+	CORBA_free (result_list);
 }
 
 static void
@@ -1152,7 +1168,7 @@ xfer_progress_ok (const GnomeVFSXferProgressInfo *info,
 							    info->bytes_total,
 							    ev);
 		break;
-	case GNOME_VFS_XFER_PHASE_XFERRING:
+	case GNOME_VFS_XFER_PHASE_COPYING:
 		if (info->bytes_copied == 0)
 			retval = GNOME_VFS_Slave_Notify_xfer_file_start
 				(notify_objref,
@@ -1172,6 +1188,7 @@ xfer_progress_ok (const GnomeVFSXferProgressInfo *info,
 		GNOME_VFS_Slave_Notify_xfer_done (notify_objref, ev);
 		retval = TRUE;
 		break;
+	case GNOME_VFS_XFER_PHASE_MOVING:
 	case GNOME_VFS_XFER_PHASE_OPENSOURCE:
 	case GNOME_VFS_XFER_PHASE_OPENTARGET:
 	case GNOME_VFS_XFER_PHASE_READSOURCE:
@@ -1179,7 +1196,7 @@ xfer_progress_ok (const GnomeVFSXferProgressInfo *info,
 	case GNOME_VFS_XFER_PHASE_CLOSESOURCE:
 	case GNOME_VFS_XFER_PHASE_CLOSETARGET:
 	case GNOME_VFS_XFER_PHASE_SETATTRIBUTES:
-	case GNOME_VFS_XFER_PHASE_UNKNOWN:
+	case GNOME_VFS_XFER_PHASE_INITIAL:
 	case GNOME_VFS_XFER_PHASE_COLLECTING:
 	default:
 		retval = TRUE;
@@ -1217,7 +1234,7 @@ xfer_progress_ovewrite (const GnomeVFSXferProgressInfo *info,
 }
 
 static gint
-xfer_progress_callback (const GnomeVFSXferProgressInfo *info,
+xfer_progress_callback (GnomeVFSXferProgressInfo *info,
 			gpointer data)
 {
 	CORBA_Environment *ev;
@@ -1374,7 +1391,8 @@ init_corba (int *argc,
 	CORBA_ORB orb;
 	PortableServer_POA poa;
 
-	orb = gnorba_CORBA_init (argc, argv, GNORBA_INIT_SERVER_FUNC, ev);
+	orb = oaf_init (*argc, argv);
+
 	if (ev->_major != CORBA_NO_EXCEPTION) {
 		error (_("Cannot initialize CORBA."));
 		return FALSE;
