@@ -26,21 +26,21 @@
 
 GnomeVFSResult	 gnome_vfs_async_cancel		(GnomeVFSAsyncHandle *handle);
 
-GnomeVFSResult	 gnome_vfs_async_open		(GnomeVFSAsyncHandle **handle_return,
+void          	 gnome_vfs_async_open		(GnomeVFSAsyncHandle **handle_return,
 						 const gchar *text_uri,
 						 GnomeVFSOpenMode open_mode,
 						 GnomeVFSAsyncOpenCallback
 						 	callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_open_uri	(GnomeVFSAsyncHandle **handle_return,
+void          	 gnome_vfs_async_open_uri	(GnomeVFSAsyncHandle **handle_return,
 						 GnomeVFSURI *uri,
 						 GnomeVFSOpenMode open_mode,
 						 GnomeVFSAsyncOpenCallback
 						 	callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_open_as_channel
+void          	 gnome_vfs_async_open_as_channel
 						(GnomeVFSAsyncHandle **handle_return,
 						 const gchar *text_uri,
 						 GnomeVFSOpenMode open_mode,
@@ -49,7 +49,7 @@ GnomeVFSResult	 gnome_vfs_async_open_as_channel
 						        callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_open_uri_as_channel
+void          	 gnome_vfs_async_open_uri_as_channel
 						(GnomeVFSAsyncHandle **handle_return,
 						 GnomeVFSURI *uri,
 						 GnomeVFSOpenMode open_mode,
@@ -58,7 +58,7 @@ GnomeVFSResult	 gnome_vfs_async_open_uri_as_channel
 						        callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_create		(GnomeVFSAsyncHandle **handle_return,
+void          	 gnome_vfs_async_create		(GnomeVFSAsyncHandle **handle_return,
 						 const gchar *text_uri,
 						 GnomeVFSOpenMode open_mode,
 						 gboolean exclusive,
@@ -67,7 +67,7 @@ GnomeVFSResult	 gnome_vfs_async_create		(GnomeVFSAsyncHandle **handle_return,
 						 	callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_create_uri	(GnomeVFSAsyncHandle **handle_return,
+void          	 gnome_vfs_async_create_uri	(GnomeVFSAsyncHandle **handle_return,
 						 GnomeVFSURI *uri,
 						 GnomeVFSOpenMode open_mode,
 						 gboolean exclusive,
@@ -75,8 +75,13 @@ GnomeVFSResult	 gnome_vfs_async_create_uri	(GnomeVFSAsyncHandle **handle_return,
 						 GnomeVFSAsyncOpenCallback
 						 	callback,
 						 gpointer callback_data);
-
-GnomeVFSResult	 gnome_vfs_async_create_as_channel
+void             gnome_vfs_async_create_symbolic_link    
+                                                (GnomeVFSAsyncHandle **handle_return,
+						 GnomeVFSURI *uri,
+						 const gchar *uri_reference,
+						 GnomeVFSAsyncOpenCallback callback,
+						 gpointer callback_data);
+void          	 gnome_vfs_async_create_as_channel
 						(GnomeVFSAsyncHandle **handle_return,
 						 const gchar *text_uri,
 						 GnomeVFSOpenMode open_mode,
@@ -86,7 +91,7 @@ GnomeVFSResult	 gnome_vfs_async_create_as_channel
 	                                                 callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_create_uri_as_channel
+void          	 gnome_vfs_async_create_uri_as_channel
 						(GnomeVFSAsyncHandle **handle_return,
 						 GnomeVFSURI *uri,
 						 GnomeVFSOpenMode open_mode,
@@ -96,37 +101,37 @@ GnomeVFSResult	 gnome_vfs_async_create_uri_as_channel
 	                                                 callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_close		(GnomeVFSAsyncHandle *handle,
+void          	 gnome_vfs_async_close		(GnomeVFSAsyncHandle *handle,
 						 GnomeVFSAsyncCloseCallback
 						 	callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_read		(GnomeVFSAsyncHandle *handle,
+void          	 gnome_vfs_async_read		(GnomeVFSAsyncHandle *handle,
 						 gpointer buffer,
 						 guint bytes,
 						 GnomeVFSAsyncReadCallback
 						 	callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_write		(GnomeVFSAsyncHandle *handle,
+void          	 gnome_vfs_async_write		(GnomeVFSAsyncHandle *handle,
 						 gconstpointer buffer,
 						 guint bytes,
 						 GnomeVFSAsyncWriteCallback
 						 	callback,
 						 gpointer callback_data);
 
-GnomeVFSResult   gnome_vfs_async_get_file_info  (GnomeVFSAsyncHandle **handle_return,
-						 const char *text_uri,
+void             gnome_vfs_async_get_file_info  (GnomeVFSAsyncHandle **handle_return,
+						 GList *uri_list, /* GnomeVFSURI* items */
 						 GnomeVFSFileInfoOptions options,
-						 const char *meta_keys[],
+						 const gchar * const meta_keys[],
 						 GnomeVFSAsyncGetFileInfoCallback callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
+void          	 gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_return,
 						 const gchar *text_uri,
 						 GnomeVFSFileInfoOptions
 						         options,
-						 const gchar *meta_keys[],
+						 const gchar * const meta_keys[],
 						 GnomeVFSDirectorySortRule
 						 	sort_rules[],
 						 gboolean reverse_order,
@@ -140,7 +145,7 @@ GnomeVFSResult	 gnome_vfs_async_load_directory (GnomeVFSAsyncHandle **handle_ret
 						         callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_load_directory_uri
+void          	 gnome_vfs_async_load_directory_uri
 					        (GnomeVFSAsyncHandle **handle_return,
 						 GnomeVFSURI *uri,
 						 GnomeVFSFileInfoOptions
@@ -159,7 +164,7 @@ GnomeVFSResult	 gnome_vfs_async_load_directory_uri
 						         callback,
 						 gpointer callback_data);
 
-GnomeVFSResult	 gnome_vfs_async_xfer		(GnomeVFSAsyncHandle **handle_return,
+GnomeVFSResult	gnome_vfs_async_xfer		(GnomeVFSAsyncHandle **handle_return,
 						 const gchar *source_dir,
 						 const GList *source_name_list,
 						 const gchar *target_dir,
@@ -171,8 +176,12 @@ GnomeVFSResult	 gnome_vfs_async_xfer		(GnomeVFSAsyncHandle **handle_return,
 						 GnomeVFSXferOverwriteMode
 						 	overwrite_mode,
 						 GnomeVFSAsyncXferProgressCallback
-						 	progress_callback,
-						 gpointer callback_data);
+						 	progress_update_callback,
+						 gpointer update_callback_data,
+						 GnomeVFSXferProgressCallback
+						 	progress_sync_callback,
+						 gpointer sync_callback_data);
+
 
 guint           gnome_vfs_async_add_status_callback
                                                 (GnomeVFSAsyncHandle *handle,
