@@ -29,7 +29,7 @@
 #include "gnome-vfs-private.h"
 
 
-struct _GnomeVFSDirectoryHandle {
+struct GnomeVFSDirectoryHandle {
 	/* URI of the directory being accessed through the handle.  */
 	GnomeVFSURI *uri;
 
@@ -49,7 +49,7 @@ struct _GnomeVFSDirectoryHandle {
 #define CHECK_IF_SUPPORTED(vfs_method, what)		\
 G_STMT_START{						\
 	if (vfs_method->what == NULL)			\
-		return GNOME_VFS_ERROR_NOTSUPPORTED;	\
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;	\
 }G_STMT_END
 
 
@@ -94,7 +94,7 @@ static GnomeVFSResult
 open_from_uri (GnomeVFSDirectoryHandle **handle,
 	       GnomeVFSURI *uri,
 	       GnomeVFSFileInfoOptions options,
-	       const gchar *meta_keys[],
+	       const gchar * const meta_keys[],
 	       const GnomeVFSDirectoryFilter *filter,
 	       GnomeVFSContext *context)
 {
@@ -102,8 +102,7 @@ open_from_uri (GnomeVFSDirectoryHandle **handle,
 	GnomeVFSResult result;
 	GList *meta_list;
 
-	if (uri->method->open_directory == NULL)
-		return GNOME_VFS_ERROR_NOTSUPPORTED;
+	CHECK_IF_SUPPORTED (uri->method, open_directory);
 
 	meta_list = gnome_vfs_string_list_from_string_array (meta_keys);
 
@@ -128,19 +127,19 @@ static GnomeVFSResult
 open (GnomeVFSDirectoryHandle **handle,
       const gchar *text_uri,
       GnomeVFSFileInfoOptions options,
-      const gchar *meta_keys[],
+      const gchar * const meta_keys[],
       const GnomeVFSDirectoryFilter *filter,
       GnomeVFSContext *context)
 {
 	GnomeVFSURI *uri;
 	GnomeVFSResult result;
 
-	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	uri = gnome_vfs_uri_new (text_uri);
 	if (uri == NULL)
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	result = open_from_uri (handle, uri, options, meta_keys, filter,
 				context);
@@ -168,11 +167,11 @@ GnomeVFSResult
 gnome_vfs_directory_open (GnomeVFSDirectoryHandle **handle,
 			  const gchar *text_uri,
 			  GnomeVFSFileInfoOptions options,
-			  const gchar *meta_keys[],
+			  const gchar * const meta_keys[],
 			  const GnomeVFSDirectoryFilter *filter)
 {
-	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	return open (handle, text_uri, options, meta_keys, filter, NULL);
 }
@@ -195,11 +194,11 @@ GnomeVFSResult
 gnome_vfs_directory_open_from_uri (GnomeVFSDirectoryHandle **handle,
 				   GnomeVFSURI *uri,
 				   GnomeVFSFileInfoOptions options,
-				   const gchar *meta_keys[],
+				   const gchar * const meta_keys[],
 				   const GnomeVFSDirectoryFilter *filter)
 {
-	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	return open_from_uri (handle, uri, options, meta_keys, filter, NULL);
 }
@@ -287,9 +286,6 @@ remove_first_reference (GList *reference_list)
 	return reference_list;
 }
 
-/* FIXME?  This leads to an O(2) algorithm, but it should not be noticeable
-   unless a *very* deep recursion (more than the depth of reasonable file
-   systems) is performed.  */
 static gboolean
 lookup_ancestor (GList *ancestors,
 		 ino_t inode,
@@ -313,7 +309,7 @@ directory_visit_internal (GnomeVFSURI *uri,
 			  const gchar *prefix,
 			  GList *ancestor_references, /* DirectoryReference */
 			  GnomeVFSFileInfoOptions info_options,
-			  const gchar *meta_keys[],
+			  const gchar * const meta_keys[],
 			  const GnomeVFSDirectoryFilter *filter,
 			  GnomeVFSDirectoryVisitOptions visit_options,
 			  GnomeVFSDirectoryVisitFunc callback,
@@ -339,7 +335,7 @@ directory_visit_internal (GnomeVFSURI *uri,
 
 		if (info->type != GNOME_VFS_FILE_TYPE_DIRECTORY) {
 			gnome_vfs_file_info_unref (info);
-			return GNOME_VFS_ERROR_NOTADIRECTORY;
+			return GNOME_VFS_ERROR_NOT_A_DIRECTORY;
 		}
 
 		ancestor_references = prepend_reference (ancestor_references,
@@ -402,7 +398,7 @@ directory_visit_internal (GnomeVFSURI *uri,
 				new_prefix = g_strconcat (prefix, info->name,
 							  "/", NULL);
 
-			new_uri = gnome_vfs_uri_append_path (uri, info->name);
+			new_uri = gnome_vfs_uri_append_file_name (uri, info->name);
 
 
 			if (GNOME_VFS_FILE_INFO_LOCAL (info))
@@ -472,13 +468,13 @@ directory_visit_internal (GnomeVFSURI *uri,
 GnomeVFSResult
 gnome_vfs_directory_visit_uri (GnomeVFSURI *uri,
 			       GnomeVFSFileInfoOptions info_options,
-			       const gchar *meta_keys[],
+			       const gchar * const meta_keys[],
 			       const GnomeVFSDirectoryFilter *filter,
 			       GnomeVFSDirectoryVisitOptions visit_options,
 			       GnomeVFSDirectoryVisitFunc callback,
 			       gpointer data)
 {
-	g_return_val_if_fail (uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	return directory_visit_internal (uri, NULL, NULL,
 					 info_options, meta_keys, filter,
@@ -505,7 +501,7 @@ gnome_vfs_directory_visit_uri (GnomeVFSURI *uri,
 GnomeVFSResult
 gnome_vfs_directory_visit (const gchar *text_uri,
 			   GnomeVFSFileInfoOptions info_options,
-			   const gchar *meta_keys[],
+			   const gchar * const meta_keys[],
 			   const GnomeVFSDirectoryFilter *filter,
 			   GnomeVFSDirectoryVisitOptions visit_options,
 			   GnomeVFSDirectoryVisitFunc callback,
@@ -514,7 +510,7 @@ gnome_vfs_directory_visit (const gchar *text_uri,
 	GnomeVFSURI *uri;
 	GnomeVFSResult result;
 
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	uri = gnome_vfs_uri_new (text_uri);
 
@@ -531,7 +527,7 @@ GnomeVFSResult
 gnome_vfs_directory_visit_files_at_uri (GnomeVFSURI *uri,
 					GList *file_list,
 					GnomeVFSFileInfoOptions info_options,
-					const gchar *meta_keys[],
+					const gchar * const meta_keys[],
 					const GnomeVFSDirectoryFilter *filter,
 					GnomeVFSDirectoryVisitOptions
 						visit_options,
@@ -542,8 +538,8 @@ gnome_vfs_directory_visit_files_at_uri (GnomeVFSURI *uri,
 	GnomeVFSResult result;
 	GList *p;
 
-	g_return_val_if_fail (uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (file_list != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (file_list != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	info = gnome_vfs_file_info_new ();
 	result = GNOME_VFS_OK;
@@ -553,7 +549,7 @@ gnome_vfs_directory_visit_files_at_uri (GnomeVFSURI *uri,
 		gboolean recurse;
 		gboolean stop;
 
-		file_uri = gnome_vfs_uri_append_path (uri, p->data);
+		file_uri = gnome_vfs_uri_append_file_name (uri, p->data);
 		gnome_vfs_get_file_info_uri (file_uri, info, info_options,
 					     meta_keys);
 
@@ -584,7 +580,7 @@ GnomeVFSResult
 gnome_vfs_directory_visit_files (const gchar *text_uri,
 				 GList *file_list,
 				 GnomeVFSFileInfoOptions info_options,
-				 const gchar *meta_keys[],
+				 const gchar * const meta_keys[],
 				 const GnomeVFSDirectoryFilter *filter,
 				 GnomeVFSDirectoryVisitOptions
 				 	visit_options,
