@@ -21,47 +21,41 @@
 
    Author: Ettore Perazzoli <ettore@comm2000.it> */
 
-#ifndef _GNOME_VFS_FILE_INFO_H
-#define _GNOME_VFS_FILE_INFO_H
+#ifndef GNOME_VFS_FILE_INFO_H
+#define GNOME_VFS_FILE_INFO_H
 
 #include "gnome-vfs.h"
 
-
-#define GNOME_VFS_FILE_INFO_SYMLINK(info)		\
-	((info)->flags & GNOME_VFS_FILE_FLAGS_SYMLINK)
-
 #define GNOME_VFS_FILE_INFO_LOCAL(info)			\
 	((info)->flags & GNOME_VFS_FILE_FLAGS_LOCAL)
 
-#define GNOME_VFS_FILE_INFO_SUID(info)			\
-	((info)->flags & GNOME_VFS_FILE_FLAGS_SUID)
-
-#define GNOME_VFS_FILE_INFO_SGID(info)			\
-	((info)->flags & GNOME_VFS_FILE_FLAGS_SGID)
-
-#define GNOME_VFS_FILE_INFO_STICKY(info)		\
-	((info)->flags & GNOME_VFS_FILE_FLAGS_STICKY)
-
-
-#define GNOME_VFS_FILE_INFO_SET_SYMLINK(info, value)			\
-	(value ? ((info)->flags |= GNOME_VFS_FILE_FLAGS_SYMLINK)	\
-	       : ((info)->flags &= ~GNOME_VFS_FILE_FLAGS_SYMLINK))
-
 #define GNOME_VFS_FILE_INFO_SET_LOCAL(info, value)			\
 	(value ? ((info)->flags |= GNOME_VFS_FILE_FLAGS_LOCAL)		\
 	       : ((info)->flags &= ~GNOME_VFS_FILE_FLAGS_LOCAL))
 
+
+
+#define GNOME_VFS_FILE_INFO_SUID(info)			\
+	((info)->permissions & GNOME_VFS_PERM_SUID)
+
+#define GNOME_VFS_FILE_INFO_SGID(info)			\
+	((info)->permissions & GNOME_VFS_PERM_SGID)
+
+#define GNOME_VFS_FILE_INFO_STICKY(info)		\
+	((info)->permissions & GNOME_VFS_PERM_STICKY)
+
+
 #define GNOME_VFS_FILE_INFO_SET_SUID(info, value)		\
-	(value ? ((info)->flags |= GNOME_VFS_FILE_FLAGS_SUID)	\
-	       : ((info)->flags &= ~GNOME_VFS_FILE_FLAGS_SUID))
+	(value ? ((info)->permissions |= GNOME_VFS_PERM_SUID)	\
+	       : ((info)->permissions &= ~GNOME_VFS_PERM_SUID))
 
 #define GNOME_VFS_FILE_INFO_SET_SGID(info, value)		\
-	(value ? ((info)->flags |= GNOME_VFS_FILE_FLAGS_SGID)	\
-	       : ((info)->flags &= ~GNOME_VFS_FILE_FLAGS_SGID))
+	(value ? ((info)->permissions |= GNOME_VFS_PERM_SGID)	\
+	       : ((info)->permissions &= ~GNOME_VFS_PERM_SGID))
 
 #define GNOME_VFS_FILE_INFO_SET_STICKY(info, value)			\
-	(value ? ((info)->flags |= GNOME_VFS_FILE_FLAGS_STICKY)		\
-	       : ((info)->flags &= ~GNOME_VFS_FILE_FLAGS_STICKY))
+	(value ? ((info)->permissions |= GNOME_VFS_PERM_STICKY)		\
+	       : ((info)->permissions &= ~GNOME_VFS_PERM_STICKY))
 
 
 
@@ -90,6 +84,9 @@ const gchar	*gnome_vfs_file_info_get_mime_type
 void		 gnome_vfs_file_info_copy 	(GnomeVFSFileInfo *dest,
 						 const GnomeVFSFileInfo *src);
 
+gboolean	 gnome_vfs_file_info_matches	(const GnomeVFSFileInfo *a,
+						 const GnomeVFSFileInfo *b);
+
 gint		 gnome_vfs_file_info_compare_for_sort
 						(const GnomeVFSFileInfo *a,
 						 const GnomeVFSFileInfo *b,
@@ -100,7 +97,9 @@ gint		 gnome_vfs_file_info_compare_for_sort_reversed
 						 const GnomeVFSFileInfo *b,
 						 const GnomeVFSDirectoryFilterType *sort_rules);
 
-/* FIXME this should go AWAY!  */
-#define gnome_vfs_file_info_destroy gnome_vfs_file_info_unref
+GList           *gnome_vfs_file_info_list_ref   (GList *list);
+GList           *gnome_vfs_file_info_list_unref (GList *list);
+GList           *gnome_vfs_file_info_list_copy  (GList *list);
+void             gnome_vfs_file_info_list_free  (GList *list);
 
-#endif                          /* _GNOME_VFS_FILE_INFO_H */
+#endif /* GNOME_VFS_FILE_INFO_H */
