@@ -31,7 +31,7 @@
 #include "gnome-vfs-private.h"
 
 
-struct _GnomeVFSHandle {
+struct GnomeVFSHandle {
 	/* URI of the file being accessed through the handle.  */
 	GnomeVFSURI *uri;
 
@@ -45,13 +45,13 @@ struct _GnomeVFSHandle {
 #define CHECK_IF_OPEN(handle)			\
 G_STMT_START{					\
 	if (handle->uri == NULL)		\
-		return GNOME_VFS_ERROR_NOTOPEN;	\
+		return GNOME_VFS_ERROR_NOT_OPEN;	\
 }G_STMT_END
 
 #define CHECK_IF_SUPPORTED(handle, what)		\
 G_STMT_START{						\
 	if (handle->uri->method->what == NULL)		\
-		return GNOME_VFS_ERROR_NOTSUPPORTED;	\
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;	\
 }G_STMT_END
 
 #define INVOKE(result, handle, what, params)		\
@@ -128,7 +128,8 @@ gnome_vfs_handle_do_close (GnomeVFSHandle *handle,
 
 	INVOKE (result, handle, close, (handle->uri->method, handle->method_handle, context));
 
-	/* Even if close has failed, we shut down the handle.  FIXME?  */
+	/* Even if close has failed, we shut down the handle. */
+	/* FIXME bugzilla.eazel.com 1169: Is this OK? */
 	gnome_vfs_handle_destroy (handle);
 
 	return result;
