@@ -48,12 +48,12 @@ gnome_vfs_open (GnomeVFSHandle **handle,
 	GnomeVFSURI *uri;
 	GnomeVFSResult result;
 
-	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	uri = gnome_vfs_uri_new (text_uri);
 	if (uri == NULL)
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	result = gnome_vfs_open_uri (handle, uri, open_mode);
 
@@ -107,12 +107,12 @@ gnome_vfs_create (GnomeVFSHandle **handle,
 	GnomeVFSURI *uri;
 	GnomeVFSResult result;
 
-	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	uri = gnome_vfs_uri_new (text_uri);
 	if (uri == NULL)
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	result = gnome_vfs_create_uri (handle, uri, open_mode, exclusive, perm);
 
@@ -243,7 +243,7 @@ GnomeVFSResult
 gnome_vfs_tell (GnomeVFSHandle *handle,
 		GnomeVFSFileSize *offset_return)
 {
-	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (handle != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	return gnome_vfs_handle_do_tell (handle, offset_return);
 }
@@ -267,7 +267,7 @@ GnomeVFSResult
 gnome_vfs_get_file_info (const gchar *text_uri,
 			 GnomeVFSFileInfo *info,
 			 GnomeVFSFileInfoOptions options,
-			 const char *meta_keys[])
+			 const char * const meta_keys[])
 {
 	GnomeVFSURI *uri;
 	GnomeVFSResult result;
@@ -275,7 +275,7 @@ gnome_vfs_get_file_info (const gchar *text_uri,
 	uri = gnome_vfs_uri_new (text_uri);
 
 	if (uri == NULL)
-		return GNOME_VFS_ERROR_NOTSUPPORTED;
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 	
 	result = gnome_vfs_get_file_info_uri(uri, info, options, meta_keys);
 	gnome_vfs_uri_unref (uri);
@@ -301,7 +301,7 @@ GnomeVFSResult
 gnome_vfs_get_file_info_uri (GnomeVFSURI *uri,
 			     GnomeVFSFileInfo *info,
 			     GnomeVFSFileInfoOptions options,
-			     const char *meta_keys[])
+			     const char * const meta_keys[])
 {
 	return gnome_vfs_get_file_info_uri_cancellable (uri, info, options,
 							meta_keys, NULL);
@@ -325,7 +325,7 @@ GnomeVFSResult
 gnome_vfs_get_file_info_from_handle (GnomeVFSHandle *handle,
 				     GnomeVFSFileInfo *info,
 				     GnomeVFSFileInfoOptions options,
-				     const char *meta_keys[])
+				     const char * const meta_keys[])
 {
 	return gnome_vfs_get_file_info_from_handle_cancellable (handle, info,
 								options,
@@ -342,7 +342,7 @@ gnome_vfs_truncate (const char *text_uri, GnomeVFSFileSize length)
 	uri = gnome_vfs_uri_new (text_uri);
 
 	if (uri == NULL)
-		return GNOME_VFS_ERROR_NOTSUPPORTED;
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 
 	result = gnome_vfs_truncate_uri(uri, length);
 	gnome_vfs_uri_unref (uri);
@@ -395,11 +395,11 @@ gnome_vfs_make_directory (const gchar *text_uri,
 	GnomeVFSResult result;
 	GnomeVFSURI *uri;
 
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	uri = gnome_vfs_uri_new (text_uri);
 	if (uri == NULL)
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	result = gnome_vfs_make_directory_for_uri (uri, perm);
 
@@ -437,11 +437,11 @@ gnome_vfs_remove_directory (const gchar *text_uri)
 	GnomeVFSResult result;
 	GnomeVFSURI *uri;
 
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	uri = gnome_vfs_uri_new (text_uri);
 	if (uri == NULL)
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	result = gnome_vfs_remove_directory_from_uri (uri);
 
@@ -465,6 +465,22 @@ gnome_vfs_unlink_from_uri (GnomeVFSURI *uri)
 	return gnome_vfs_unlink_from_uri_cancellable (uri, NULL);
 }
 
+/**
+ * gnome_vfs_create_symbolic_link:
+ * @uri: URI to create a link at
+ * @target_reference: URI "reference" to point the link to (URI or relative path)
+ *
+ * Creates a symbolic link, or eventually, a URI link (as necessary) 
+ * at @uri pointing to @target_reference
+ *
+ * Return value: An integer representing the result of the operation
+ **/
+GnomeVFSResult
+gnome_vfs_create_symbolic_link (GnomeVFSURI *uri, const gchar *target_reference)
+{
+	return gnome_vfs_create_symbolic_link_cancellable (uri, target_reference, NULL);
+}
+
 /**
  * gnome_vfs_unlink_from_uri:
  * @text_uri: URI of the file to be unlinked
@@ -479,11 +495,11 @@ gnome_vfs_unlink (const gchar *text_uri)
 	GnomeVFSResult result;
 	GnomeVFSURI *uri;
 
-	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	uri = gnome_vfs_uri_new (text_uri);
 	if (uri == NULL)
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	result = gnome_vfs_unlink_from_uri (uri);
 
@@ -495,53 +511,54 @@ gnome_vfs_unlink (const gchar *text_uri)
 
 /**
  * gnome_vfs_move_uri:
- * @old: Source URI
- * @new: Destination URI
+ * @old_uri: Source URI
+ * @new_uri: Destination URI
  * 
- * Move a file from URI @old to @new.  This will only work if @old and @new are
- * on the same file system.  Otherwise, you it is necessary to use the more
- * general %gnome_vfs_xfer_uri() function.
+ * Move a file from URI @old_uri to @new_uri.  This will only work if @old_uri 
+ * and @new_uri are on the same file system.  Otherwise, it is necessary 
+ * to use the more general %gnome_vfs_xfer_uri() function.
  * 
  * Return value: An integer representing the result of the operation.
  **/
 GnomeVFSResult
-gnome_vfs_move_uri (GnomeVFSURI *old,
-		    GnomeVFSURI *new,
+gnome_vfs_move_uri (GnomeVFSURI *old_uri,
+		    GnomeVFSURI *new_uri,
 		    gboolean force_replace)
 {
-	return gnome_vfs_move_uri_cancellable (old, new, force_replace, NULL);
+	return gnome_vfs_move_uri_cancellable (old_uri, new_uri, 
+					       force_replace, NULL);
 }
 
 /**
  * gnome_vfs_move:
- * @old: Source URI
- * @new: Destination URI
+ * @old_text_uri: Source URI
+ * @new_text_uri: Destination URI
  * 
- * Move a file from URI @old to @new.  This will only work if @old and @new are
- * on the same file system.  Otherwise, you it is necessary to use the more
- * general %gnome_vfs_xfer_uri() function.
+ * Move a file from URI @old_text_uri to @new_text_uri.  This will only work 
+ * if @old_text_uri and @new_text_uri are on the same file system.  Otherwise,
+ * it is necessary to use the more general %gnome_vfs_xfer_uri() function.
  * 
  * Return value: An integer representing the result of the operation.
  **/
 GnomeVFSResult
-gnome_vfs_move (const gchar *old,
-		const gchar *new,
+gnome_vfs_move (const gchar *old_text_uri,
+		const gchar *new_text_uri,
 		gboolean force_replace)
 {
 	GnomeVFSURI *old_uri, *new_uri;
 	GnomeVFSResult retval;
 
-	g_return_val_if_fail (old != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (new != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (old_text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (new_text_uri != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
-	old_uri = gnome_vfs_uri_new (old);
+	old_uri = gnome_vfs_uri_new (old_text_uri);
 	if (old_uri == NULL)
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
-	new_uri = gnome_vfs_uri_new (new);
+	new_uri = gnome_vfs_uri_new (new_text_uri);
 	if (new_uri == NULL) {
 		gnome_vfs_uri_unref (old_uri);
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 	}
 
 	retval = gnome_vfs_move_uri (old_uri, new_uri, force_replace);
@@ -592,20 +609,20 @@ gnome_vfs_check_same_fs (const gchar *a,
 	GnomeVFSURI *a_uri, *b_uri;
 	GnomeVFSResult retval;
 
-	g_return_val_if_fail (a != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (b != NULL, GNOME_VFS_ERROR_BADPARAMS);
-	g_return_val_if_fail (same_fs_return != NULL, GNOME_VFS_ERROR_BADPARAMS);
+	g_return_val_if_fail (a != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (b != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
+	g_return_val_if_fail (same_fs_return != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
 	*same_fs_return = FALSE;
 
 	a_uri = gnome_vfs_uri_new (a);
 	if (a_uri == NULL)
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	b_uri = gnome_vfs_uri_new (b);
 	if (b_uri == NULL) {
 		gnome_vfs_uri_unref (a_uri);
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 	}
 
 	retval = gnome_vfs_check_same_fs_uris (a_uri, b_uri, same_fs_return);
@@ -656,7 +673,7 @@ gnome_vfs_set_file_info (const gchar *text_uri,
 
 	uri = gnome_vfs_uri_new (text_uri);
 	if (uri == NULL)
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	result = gnome_vfs_set_file_info_uri (uri, info, mask);
 
