@@ -46,7 +46,7 @@
 
 
 GList *
-gnome_vfs_string_list_from_string_array (const gchar *array[])
+gnome_vfs_string_list_from_string_array (const gchar * const array[])
 {
 	GList *list;
 	guint i;
@@ -214,12 +214,9 @@ shut_down_file_descriptors (void)
 		close (i);
 }
 
-/* FIXME I am not sure the following stuff should be here.  This is a bit
-   messy.  */
-
 pid_t
 gnome_vfs_forkexec (const gchar *file_name,
-		    gchar *const argv[],
+		    const gchar * const argv[],
 		    GnomeVFSProcessOptions options,
 		    GnomeVFSProcessInitFunc init_func,
 		    gpointer init_data)
@@ -235,9 +232,9 @@ gnome_vfs_forkexec (const gchar *file_name,
 		if (options & GNOME_VFS_PROCESS_CLOSEFDS)
 			shut_down_file_descriptors ();
 		if (options & GNOME_VFS_PROCESS_USEPATH)
-			execvp (file_name, argv);
+			execvp (file_name, (char **) argv);
 		else
-			execv (file_name, argv);
+			execv (file_name, (char **) argv);
 		_exit (1);
 	}
 
@@ -260,7 +257,7 @@ gnome_vfs_forkexec (const gchar *file_name,
  **/
 GnomeVFSProcessResult
 gnome_vfs_process_run_cancellable (const gchar *file_name,
-				   gchar *const argv[],
+				   const gchar * const argv[],
 				   GnomeVFSProcessOptions options,
 				   GnomeVFSCancellation *cancellation,
 				   guint *exit_value)
@@ -340,7 +337,7 @@ gnome_vfs_create_temp (const gchar *prefix,
 			return GNOME_VFS_OK;
 		}
 
-		if (result != GNOME_VFS_ERROR_FILEEXISTS) {
+		if (result != GNOME_VFS_ERROR_FILE_EXISTS) {
 			*name_return = NULL;
 			*handle_return = NULL;
 			return result;
@@ -382,11 +379,11 @@ check_end (const gchar *p)
 {
 	if (!p)
 		return 0;
-	while (isspace (*p))
+	while (isspace ((unsigned char)*p))
 		++p;
 	if (!*p
 	    || (p[0] == 'G' && p[1] == 'M' && p[2] == 'T')
-	    || ((p[0] == '+' || p[1] == '-') && isdigit (p[1])))
+	    || ((p[0] == '+' || p[1] == '-') && isdigit ((unsigned char)p[1])))
 		return 1;
 	else
 		return 0;
@@ -469,4 +466,4 @@ gnome_vfs_atotm (const gchar *time_string,
 
 	/* Failure.  */
 	return FALSE;
-}
\ No newline at end of file
+}
