@@ -21,13 +21,27 @@
    Author: Ettore Perazzoli <ettore@gnu.org>
 */
 
-#ifndef _GNOME_VFS_TYPES_H
-#define _GNOME_VFS_TYPES_H
+#ifndef GNOME_VFS_TYPES_H
+#define GNOME_VFS_TYPES_H
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <time.h>
-#include <glibconfig.h>
+#include <glib.h>
+
+/*
+ * This defines GnomeVFSFileSize and GnomeVFSFileOffset
+ *
+ * It also defines GNOME_VFS_SIZE_IS_<type> and GNOME_VFS_OFFSET_IS_<type>
+ * where type is INT, UNSIGNED_INT, LONG, UNSIGNED_LONG, LONG_LONG
+ * or UNSIGNED_LONG_LONG.  Note that size is always unsigned and offset
+ * is always signed.
+ *
+ * It also defines GNOME_VFS_SIZE_FORMAT_STR and GNOME_VFS_OFFSET_FORMAT_STR
+ * which is the string representation to be used in printf style expressions.
+ * This is without the %, so for example for long it would be "ld"
+ */
+#include <gnome-vfs-file-size.h>
 
 
 /* Basic enumerations.  */
@@ -35,75 +49,78 @@
 /* IMPORTANT NOTICE: If you add error types here, please also add the
    corresponsing descriptions in `gnome-vfs-result.c'.  Moreover, *always* add
    new values at the end of the list, and *never* remove values.  */
-enum _GnomeVFSResult {
+enum GnomeVFSResult {
 	GNOME_VFS_OK,
-	GNOME_VFS_ERROR_NOTFOUND,
+	GNOME_VFS_ERROR_NOT_FOUND,
 	GNOME_VFS_ERROR_GENERIC,
 	GNOME_VFS_ERROR_INTERNAL,
-	GNOME_VFS_ERROR_BADPARAMS,
-	GNOME_VFS_ERROR_NOTSUPPORTED,
+	GNOME_VFS_ERROR_BAD_PARAMETERS,
+	GNOME_VFS_ERROR_NOT_SUPPORTED,
 	GNOME_VFS_ERROR_IO,
-	GNOME_VFS_ERROR_CORRUPTEDDATA,
-	GNOME_VFS_ERROR_WRONGFORMAT,
-	GNOME_VFS_ERROR_BADFILE,
-	GNOME_VFS_ERROR_TOOBIG,
-	GNOME_VFS_ERROR_NOSPACE,
-	GNOME_VFS_ERROR_READONLY,
-	GNOME_VFS_ERROR_INVALIDURI,
-	GNOME_VFS_ERROR_NOTOPEN,
-	GNOME_VFS_ERROR_INVALIDOPENMODE,
-	GNOME_VFS_ERROR_ACCESSDENIED,
-	GNOME_VFS_ERROR_TOOMANYOPENFILES,
+	GNOME_VFS_ERROR_CORRUPTED_DATA,
+	GNOME_VFS_ERROR_WRONG_FORMAT,
+	GNOME_VFS_ERROR_BAD_FILE,
+	GNOME_VFS_ERROR_TOO_BIG,
+	GNOME_VFS_ERROR_NO_SPACE,
+	GNOME_VFS_ERROR_READ_ONLY,
+	GNOME_VFS_ERROR_INVALID_URI,
+	GNOME_VFS_ERROR_NOT_OPEN,
+	GNOME_VFS_ERROR_INVALID_OPEN_MODE,
+	GNOME_VFS_ERROR_ACCESS_DENIED,
+	GNOME_VFS_ERROR_TOO_MANY_OPEN_FILES,
 	GNOME_VFS_ERROR_EOF,
-	GNOME_VFS_ERROR_NOTADIRECTORY,
-	GNOME_VFS_ERROR_INPROGRESS,
+	GNOME_VFS_ERROR_NOT_A_DIRECTORY,
+	GNOME_VFS_ERROR_IN_PROGRESS,
 	GNOME_VFS_ERROR_INTERRUPTED,
-	GNOME_VFS_ERROR_FILEEXISTS,
+	GNOME_VFS_ERROR_FILE_EXISTS,
 	GNOME_VFS_ERROR_LOOP,
-	GNOME_VFS_ERROR_NOTPERMITTED,
-	GNOME_VFS_ERROR_ISDIRECTORY,
-	GNOME_VFS_ERROR_NOMEM,
-	GNOME_VFS_ERROR_HOSTNOTFOUND,
-	GNOME_VFS_ERROR_INVALIDHOSTNAME,
-	GNOME_VFS_ERROR_HOSTHASNOADDRESS,
-	GNOME_VFS_ERROR_LOGINFAILED,
+	GNOME_VFS_ERROR_NOT_PERMITTED,
+	GNOME_VFS_ERROR_IS_DIRECTORY,
+	GNOME_VFS_ERROR_NO_MEMORY,
+	GNOME_VFS_ERROR_HOST_NOT_FOUND,
+	GNOME_VFS_ERROR_INVALID_HOST_NAME,
+	GNOME_VFS_ERROR_HOST_HAS_NO_ADDRESS,
+	GNOME_VFS_ERROR_LOGIN_FAILED,
 	GNOME_VFS_ERROR_CANCELLED,
-	GNOME_VFS_ERROR_DIRECTORYBUSY,
-	GNOME_VFS_ERROR_DIRECTORYNOTEMPTY,
-	GNOME_VFS_ERROR_TOOMANYLINKS,
-	GNOME_VFS_ERROR_READONLYFS,
-	GNOME_VFS_ERROR_NOTSAMEFS,
-	GNOME_VFS_ERROR_NAMETOOLONG,
+	GNOME_VFS_ERROR_DIRECTORY_BUSY,
+	GNOME_VFS_ERROR_DIRECTORY_NOT_EMPTY,
+	GNOME_VFS_ERROR_TOO_MANY_LINKS,
+	GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM,
+	GNOME_VFS_ERROR_NOT_SAME_FILE_SYSTEM,
+	GNOME_VFS_ERROR_NAME_TOO_LONG,
 	GNOME_VFS_NUM_ERRORS
 };
-typedef enum _GnomeVFSResult GnomeVFSResult;
+typedef enum GnomeVFSResult GnomeVFSResult;
 
 /* Open mode.  If you don't set `GNOME_VFS_OPEN_RANDOM', you have to access the
    file sequentially.  */
-enum _GnomeVFSOpenMode {
+enum GnomeVFSOpenMode {
 	GNOME_VFS_OPEN_NONE = 0,
 	GNOME_VFS_OPEN_READ = 1 << 0,
 	GNOME_VFS_OPEN_WRITE = 1 << 1,
 	GNOME_VFS_OPEN_RANDOM = 1 << 2
 };
-typedef enum _GnomeVFSOpenMode GnomeVFSOpenMode;
+typedef enum GnomeVFSOpenMode GnomeVFSOpenMode;
 
 /* The file type.  */
-enum _GnomeVFSFileType {
+enum GnomeVFSFileType {
 	GNOME_VFS_FILE_TYPE_UNKNOWN,
 	GNOME_VFS_FILE_TYPE_REGULAR,
 	GNOME_VFS_FILE_TYPE_DIRECTORY,
 	GNOME_VFS_FILE_TYPE_FIFO,
 	GNOME_VFS_FILE_TYPE_SOCKET,
-	GNOME_VFS_FILE_TYPE_CHARDEVICE,
-	GNOME_VFS_FILE_TYPE_BLOCKDEVICE,
-	GNOME_VFS_FILE_TYPE_BROKENSYMLINK
+	GNOME_VFS_FILE_TYPE_CHARACTER_DEVICE,
+	GNOME_VFS_FILE_TYPE_BLOCK_DEVICE,
+	GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK
 };
-typedef enum _GnomeVFSFileType GnomeVFSFileType;
+typedef enum GnomeVFSFileType GnomeVFSFileType;
 
 /* File permissions.  These are the same as the Unix ones, but we wrap them
    into a nicer VFS-like enum.  */
-enum _GnomeVFSFilePermissions {
+enum GnomeVFSFilePermissions {
+	GNOME_VFS_PERM_SUID = S_ISUID,
+	GNOME_VFS_PERM_SGID = S_ISGID,	
+	GNOME_VFS_PERM_STICKY = 01000,	/* S_ISVTX not defined on all systems */
 	GNOME_VFS_PERM_USER_READ = S_IRUSR,
 	GNOME_VFS_PERM_USER_WRITE = S_IWUSR,
 	GNOME_VFS_PERM_USER_EXEC = S_IXUSR,
@@ -117,35 +134,27 @@ enum _GnomeVFSFilePermissions {
 	GNOME_VFS_PERM_OTHER_EXEC = S_IXOTH,
 	GNOME_VFS_PERM_OTHER_ALL = S_IROTH | S_IWOTH | S_IXOTH
 };
-typedef enum _GnomeVFSFilePermissions GnomeVFSFilePermissions;
+typedef enum GnomeVFSFilePermissions GnomeVFSFilePermissions;
 
 /* This is used to specify the start position for seek operations.  */
-enum _GnomeVFSSeekPosition {
+enum GnomeVFSSeekPosition {
 	GNOME_VFS_SEEK_START,
 	GNOME_VFS_SEEK_CURRENT,
 	GNOME_VFS_SEEK_END
 };
-typedef enum _GnomeVFSSeekPosition GnomeVFSSeekPosition;
+typedef enum GnomeVFSSeekPosition GnomeVFSSeekPosition;
 
 
 /* Basic types.  */
 
-#ifdef G_HAVE_GINT64
-typedef guint64 GnomeVFSFileSize;
-typedef gint64  GnomeVFSFileOffset;
-#else
-typedef gulong GnomeVFSFileSize;
-typedef glong  GnomeVFSFileOffset;
-#endif
-
 /* A file handle.  */
-typedef struct _GnomeVFSHandle GnomeVFSHandle;
+typedef struct GnomeVFSHandle GnomeVFSHandle;
 
 /* Structure describing an access method.  */
-typedef struct _GnomeVFSMethod GnomeVFSMethod;
+typedef struct GnomeVFSMethod GnomeVFSMethod;
 
 /* This describes a URI element.  */
-struct _GnomeVFSURI {
+struct GnomeVFSURI {
 	/* Reference count.  */
 	guint ref_count;
 
@@ -161,14 +170,14 @@ struct _GnomeVFSURI {
 	GnomeVFSMethod *method;
 
 	/* Pointer to the parent element, or NULL for toplevel elements.  */
-	struct _GnomeVFSURI *parent;
+	struct GnomeVFSURI *parent;
 };
-typedef struct _GnomeVFSURI GnomeVFSURI;
+typedef struct GnomeVFSURI GnomeVFSURI;
 
 /* This is the toplevel URI element.  A toplevel method implementations should
    cast the `GnomeVFSURI' argument to this type to get the additional host/auth
    information.  If any of the elements is 0, it is unspecified.  */
-struct _GnomeVFSToplevelURI {
+struct GnomeVFSToplevelURI {
 	/* Base object.  */
 	GnomeVFSURI uri;
 
@@ -179,12 +188,15 @@ struct _GnomeVFSToplevelURI {
 	/* Authorization information.  */
 	gchar *user_name;
 	gchar *password;
+
+	/* The parent URN, if it exists */
+	gchar *urn;
 };
-typedef struct _GnomeVFSToplevelURI GnomeVFSToplevelURI;
+typedef struct GnomeVFSToplevelURI GnomeVFSToplevelURI;
 
 /* This is used for hiding information when transforming the GnomeVFSURI into a
    string.  */
-enum _GnomeVFSURIHideOptions {
+enum GnomeVFSURIHideOptions {
 	GNOME_VFS_URI_HIDE_NONE = 0,
 	GNOME_VFS_URI_HIDE_USER_NAME = 1 << 0,
 	GNOME_VFS_URI_HIDE_PASSWORD = 1 << 1,
@@ -192,34 +204,26 @@ enum _GnomeVFSURIHideOptions {
 	GNOME_VFS_URI_HIDE_HOST_PORT = 1 << 3,
 	GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD = 1 << 4
 };
-typedef enum _GnomeVFSURIHideOptions GnomeVFSURIHideOptions;
+typedef enum GnomeVFSURIHideOptions GnomeVFSURIHideOptions;
 
 
 /* File information, a la stat(2).  */
-/* FIXME: Private stuff.  */
+/* FIXME bugzilla.eazel.com 1202: Some of GnomeVFSFileInfo could be private.  */
 
-struct _GnomeVFSFileMetadata {
+struct GnomeVFSFileMetadata {
 	gchar *key;
 	gpointer *value;
 	guint value_size;
 };
-typedef struct _GnomeVFSFileMetadata GnomeVFSFileMetadata;
+typedef struct GnomeVFSFileMetadata GnomeVFSFileMetadata;
 
 /* File flags.  */
-enum _GnomeVFSFileFlags {
+enum GnomeVFSFileFlags {
 	GNOME_VFS_FILE_FLAGS_NONE = 0,
-	/* Whether the file is a symlink.  */
-	GNOME_VFS_FILE_FLAGS_SYMLINK = 1 << 0,
 	/* Whether the file is on a local file system.  */
 	GNOME_VFS_FILE_FLAGS_LOCAL = 1 << 1,
-	/* Whether the file has the SUID bit set.  */
-	GNOME_VFS_FILE_FLAGS_SUID = 1 << 2,
-	/* Whether the file has the SGID bit set.  */
-	GNOME_VFS_FILE_FLAGS_SGID = 1 << 3,
-	/* Whether the file has the sticky bit set.  */
-	GNOME_VFS_FILE_FLAGS_STICKY = 1 << 4
 };
-typedef enum _GnomeVFSFileFlags GnomeVFSFileFlags;
+typedef enum GnomeVFSFileFlags GnomeVFSFileFlags;
 
 /* Flags indicating what fields in a GnomeVFSFileInfo struct are valid. 
 
@@ -228,7 +232,7 @@ typedef enum _GnomeVFSFileFlags GnomeVFSFileFlags;
    it shouldn't be accessed directly anyway.
  */
 
-enum _GnomeVFSFileInfoFields {
+enum GnomeVFSFileInfoFields {
 	GNOME_VFS_FILE_INFO_FIELDS_NONE = 0,
 	GNOME_VFS_FILE_INFO_FIELDS_TYPE = 1 << 0,
 	GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS = 1 << 1,
@@ -245,9 +249,9 @@ enum _GnomeVFSFileInfoFields {
 	GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME = 1 << 12,
 	GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE = 1 << 13
 };
-typedef enum _GnomeVFSFileInfoFields GnomeVFSFileInfoFields;
+typedef enum GnomeVFSFileInfoFields GnomeVFSFileInfoFields;
 
-struct _GnomeVFSFileInfo {
+struct GnomeVFSFileInfo {
 	/* Base name of the file (no path).  */
 	gchar *name;
 
@@ -304,34 +308,42 @@ struct _GnomeVFSFileInfo {
 
 	guint refcount;
 };
-typedef struct _GnomeVFSFileInfo GnomeVFSFileInfo;
+typedef struct GnomeVFSFileInfo GnomeVFSFileInfo;
 
-enum _GnomeVFSFileInfoOptions {
-	GNOME_VFS_FILE_INFO_DEFAULT = 0, /* FIXME name sucks */
-	GNOME_VFS_FILE_INFO_GETMIMETYPE = 1 << 0,
-	GNOME_VFS_FILE_INFO_FASTMIMETYPE = 1 << 1,
-	GNOME_VFS_FILE_INFO_FOLLOWLINKS = 1 << 2
+enum GnomeVFSFileInfoOptions {
+	GNOME_VFS_FILE_INFO_DEFAULT = 0, /* FIXME bugzilla.eazel.com 1203: name sucks */
+	GNOME_VFS_FILE_INFO_GET_MIME_TYPE = 1 << 0,
+	GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE = 1 << 1,
+	GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE = 1 << 2,
+	GNOME_VFS_FILE_INFO_FOLLOW_LINKS = 1 << 3
 };
-typedef enum _GnomeVFSFileInfoOptions GnomeVFSFileInfoOptions;
+typedef enum GnomeVFSFileInfoOptions GnomeVFSFileInfoOptions;
 
-enum _GnomeVFSSetFileInfoMask {
+enum GnomeVFSSetFileInfoMask {
 	GNOME_VFS_SET_FILE_INFO_NONE = 0,
 	GNOME_VFS_SET_FILE_INFO_NAME = 1 << 0,
 	GNOME_VFS_SET_FILE_INFO_PERMISSIONS = 1 << 1,
 	GNOME_VFS_SET_FILE_INFO_OWNER = 1 << 2,
 	GNOME_VFS_SET_FILE_INFO_TIME = 1 << 3
 };
-typedef enum _GnomeVFSSetFileInfoMask GnomeVFSSetFileInfoMask;
+typedef enum GnomeVFSSetFileInfoMask GnomeVFSSetFileInfoMask;
 
-
 /* Directory stuff.  */
 
-typedef struct _GnomeVFSDirectoryList GnomeVFSDirectoryList;
+enum _GnomeVFSFindDirectoryKind {
+
+	GNOME_VFS_DIRECTORY_KIND_DESKTOP = 1000,
+	GNOME_VFS_DIRECTORY_KIND_TRASH = 1001
+};
+typedef enum _GnomeVFSFindDirectoryKind GnomeVFSFindDirectoryKind;
+
+typedef struct GnomeVFSDirectoryList GnomeVFSDirectoryList;
+
 typedef gpointer GnomeVFSDirectoryListPosition;
 
 #define GNOME_VFS_DIRECTORY_LIST_POSITION_NONE NULL
 
-enum _GnomeVFSDirectorySortRule {
+enum GnomeVFSDirectorySortRule {
 	GNOME_VFS_DIRECTORY_SORT_NONE,
 	GNOME_VFS_DIRECTORY_SORT_DIRECTORYFIRST,
 	GNOME_VFS_DIRECTORY_SORT_BYNAME,
@@ -343,16 +355,16 @@ enum _GnomeVFSDirectorySortRule {
 	GNOME_VFS_DIRECTORY_SORT_BYCTIME,
 	GNOME_VFS_DIRECTORY_SORT_BYMIMETYPE
 };
-typedef enum _GnomeVFSDirectorySortRule GnomeVFSDirectorySortRule;
+typedef enum GnomeVFSDirectorySortRule GnomeVFSDirectorySortRule;
 
-enum _GnomeVFSDirectoryFilterType {
+enum GnomeVFSDirectoryFilterType {
 	GNOME_VFS_DIRECTORY_FILTER_NONE,
 	GNOME_VFS_DIRECTORY_FILTER_SHELLPATTERN,
 	GNOME_VFS_DIRECTORY_FILTER_REGEXP
 };
-typedef enum _GnomeVFSDirectoryFilterType GnomeVFSDirectoryFilterType;
+typedef enum GnomeVFSDirectoryFilterType GnomeVFSDirectoryFilterType;
 
-enum _GnomeVFSDirectoryFilterOptions {
+enum GnomeVFSDirectoryFilterOptions {
 	GNOME_VFS_DIRECTORY_FILTER_DEFAULT = 0,
 	GNOME_VFS_DIRECTORY_FILTER_NODIRS = 1 << 0,
 	GNOME_VFS_DIRECTORY_FILTER_DIRSONLY = 1 << 1,
@@ -362,9 +374,9 @@ enum _GnomeVFSDirectoryFilterOptions {
 	GNOME_VFS_DIRECTORY_FILTER_NOSELFDIR = 1 << 5,
 	GNOME_VFS_DIRECTORY_FILTER_NOPARENTDIR = 1 << 6
 };
-typedef enum _GnomeVFSDirectoryFilterOptions GnomeVFSDirectoryFilterOptions;
+typedef enum GnomeVFSDirectoryFilterOptions GnomeVFSDirectoryFilterOptions;
 
-enum _GnomeVFSDirectoryFilterNeeds {
+enum GnomeVFSDirectoryFilterNeeds {
 	GNOME_VFS_DIRECTORY_FILTER_NEEDS_NOTHING = 0,
 	GNOME_VFS_DIRECTORY_FILTER_NEEDS_NAME = 1 << 0,
 	GNOME_VFS_DIRECTORY_FILTER_NEEDS_TYPE = 1 << 1,
@@ -372,16 +384,16 @@ enum _GnomeVFSDirectoryFilterNeeds {
 	GNOME_VFS_DIRECTORY_FILTER_NEEDS_MIMETYPE = 1 << 3,
 	GNOME_VFS_DIRECTORY_FILTER_NEEDS_METADATA = 1 << 4
 };
-typedef enum _GnomeVFSDirectoryFilterNeeds GnomeVFSDirectoryFilterNeeds;
+typedef enum GnomeVFSDirectoryFilterNeeds GnomeVFSDirectoryFilterNeeds;
 
-enum _GnomeVFSDirectoryVisitOptions {
+enum GnomeVFSDirectoryVisitOptions {
 	GNOME_VFS_DIRECTORY_VISIT_DEFAULT = 0,
 	GNOME_VFS_DIRECTORY_VISIT_SAMEFS = 1 << 0,
 	GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK = 1 << 1
 };
-typedef enum _GnomeVFSDirectoryVisitOptions GnomeVFSDirectoryVisitOptions;
+typedef enum GnomeVFSDirectoryVisitOptions GnomeVFSDirectoryVisitOptions;
 
-typedef struct _GnomeVFSDirectoryFilter GnomeVFSDirectoryFilter;
+typedef struct GnomeVFSDirectoryFilter GnomeVFSDirectoryFilter;
 
 typedef gboolean (* GnomeVFSDirectoryFilterFunc) (const GnomeVFSFileInfo *info,
 						  gpointer data);
@@ -395,31 +407,39 @@ typedef gboolean (* GnomeVFSDirectoryVisitFunc)	 (const gchar *rel_path,
 						  gboolean *recurse);
 
 
-/* Xfer options.  */
-enum _GnomeVFSXferOptions {
+/* Xfer options.  
+ * FIXME bugzilla.eazel.com 1205:
+ * Split these up into xfer options and xfer actions
+ */
+enum GnomeVFSXferOptions {
 	GNOME_VFS_XFER_DEFAULT = 0,
-	GNOME_VFS_XFER_PRESERVE = 1 << 0,
-	GNOME_VFS_XFER_FOLLOWLINKS = 1 << 1,
-	GNOME_VFS_XFER_WITHPARENTS = 1 << 2,
+	GNOME_VFS_XFER_UNUSED_1 = 1 << 0,
+	GNOME_VFS_XFER_FOLLOW_LINKS = 1 << 1,
+	GNOME_VFS_XFER_UNUSED_2 = 1 << 2,
 	GNOME_VFS_XFER_RECURSIVE = 1 << 3,
 	GNOME_VFS_XFER_SAMEFS = 1 << 4,
-	GNOME_VFS_XFER_SPARSE_ALWAYS = 1 << 5,
-	GNOME_VFS_XFER_SPARSE_NEVER = 1 << 6,
-	GNOME_VFS_XFER_UPDATEMODE = 1 << 7,
-	GNOME_VFS_XFER_REMOVESOURCE = 1 << 8
+	GNOME_VFS_XFER_DELETE_ITEMS = 1 << 5,
+	GNOME_VFS_XFER_EMPTY_DIRECTORIES = 1 << 6,
+	GNOME_VFS_XFER_NEW_UNIQUE_DIRECTORY = 1 << 7,
+	GNOME_VFS_XFER_REMOVESOURCE = 1 << 8,
+	GNOME_VFS_XFER_USE_UNIQUE_NAMES = 1 << 9
+	
 };
-typedef enum _GnomeVFSXferOptions GnomeVFSXferOptions;
+typedef enum GnomeVFSXferOptions GnomeVFSXferOptions;
 
 /* Progress status, to be reported to the caller of the transfer operation.  */
-enum _GnomeVFSXferProgressStatus {
+enum GnomeVFSXferProgressStatus {
 	GNOME_VFS_XFER_PROGRESS_STATUS_OK = 0,
 	GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR = 1,
-	GNOME_VFS_XFER_PROGRESS_STATUS_OVERWRITE = 2
+	GNOME_VFS_XFER_PROGRESS_STATUS_OVERWRITE = 2,
+	/* during the duplicate status the progress callback is asked to
+	   supply a new unique name */
+	GNOME_VFS_XFER_PROGRESS_STATUS_DUPLICATE = 3
 };
-typedef enum _GnomeVFSXferProgressStatus GnomeVFSXferProgressStatus;
+typedef enum GnomeVFSXferProgressStatus GnomeVFSXferProgressStatus;
 
 /* The different ways to deal with overwriting during a transfer operation.  */
-enum _GnomeVFSXferOverwriteMode {
+enum GnomeVFSXferOverwriteMode {
 	/* Interrupt transferring with an error (GNOME_VFS_ERROR_FILEEXISTS).  */
 	GNOME_VFS_XFER_OVERWRITE_MODE_ABORT = 0,
 	/* Invoke the progress callback with a
@@ -430,20 +450,20 @@ enum _GnomeVFSXferOverwriteMode {
 	/* Ignore files silently.  */
 	GNOME_VFS_XFER_OVERWRITE_MODE_SKIP = 3
 };
-typedef enum _GnomeVFSXferOverwriteMode GnomeVFSXferOverwriteMode;
+typedef enum GnomeVFSXferOverwriteMode GnomeVFSXferOverwriteMode;
 
 /* This defines the actions to perform before a file is being overwritten
    (i.e., these are the answers that can be given to a replace query).  */
-enum _GnomeVFSXferOverwriteAction {
+enum GnomeVFSXferOverwriteAction {
 	GNOME_VFS_XFER_OVERWRITE_ACTION_ABORT = 0,
 	GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE = 1,
 	GNOME_VFS_XFER_OVERWRITE_ACTION_REPLACE_ALL = 2,
 	GNOME_VFS_XFER_OVERWRITE_ACTION_SKIP = 3,
 	GNOME_VFS_XFER_OVERWRITE_ACTION_SKIP_ALL = 4,
 };
-typedef enum _GnomeVFSXferOverwriteAction GnomeVFSXferOverwriteAction;
+typedef enum GnomeVFSXferOverwriteAction GnomeVFSXferOverwriteAction;
 
-enum _GnomeVFSXferErrorMode {
+enum GnomeVFSXferErrorMode {
 	/* Interrupt transferring with an error (code returned is code of the
            operation that has caused the error).  */
 	GNOME_VFS_XFER_ERROR_MODE_ABORT = 0,
@@ -451,11 +471,11 @@ enum _GnomeVFSXferErrorMode {
 	   `GNOME_VFS_XFER_PROGRESS_STATUS_VFSERROR' status code. */
 	GNOME_VFS_XFER_ERROR_MODE_QUERY = 1,
 };
-typedef enum _GnomeVFSXferErrorMode GnomeVFSXferErrorMode;
+typedef enum GnomeVFSXferErrorMode GnomeVFSXferErrorMode;
 
 /* This defines the possible actions to be performed after an error has
    occurred.  */
-enum _GnomeVFSXferErrorAction {
+enum GnomeVFSXferErrorAction {
 	/* Interrupt operation and return `GNOME_VFS_ERROR_INTERRUPTED'.  */
 	GNOME_VFS_XFER_ERROR_ACTION_ABORT = 0,
 	/* Try the same operation again.  */
@@ -463,25 +483,27 @@ enum _GnomeVFSXferErrorAction {
 	/* Skip this file and continue normally.  */
 	GNOME_VFS_XFER_ERROR_ACTION_SKIP = 2
 };
-typedef enum _GnomeVFSXferErrorAction GnomeVFSXferErrorAction;
+typedef enum GnomeVFSXferErrorAction GnomeVFSXferErrorAction;
 
 /* This specifies the current phase in the transfer operation.  Phases whose
    comments are marked with `(*)' are always reported in "normal" (i.e. no
    error) condition; the other ones are only reported if an error happens in
    that specific phase.  */
-enum _GnomeVFSXferPhase {
-	/* Unknown phase */
-	GNOME_VFS_XFER_PHASE_UNKNOWN,
+enum GnomeVFSXferPhase {
+	/* Initial phase */
+	GNOME_VFS_XFER_PHASE_INITIAL,
 	/* Collecting file list */
 	GNOME_VFS_XFER_PHASE_COLLECTING,
 	/* File list collected (*) */
 	GNOME_VFS_XFER_PHASE_READYTOGO,
 	/* Opening source file for reading */
 	GNOME_VFS_XFER_PHASE_OPENSOURCE,
-	/* Creating target file */
+	/* Creating target file for copy */
 	GNOME_VFS_XFER_PHASE_OPENTARGET,
-	/* Ready to transfer (more) data from source to target (*) */
-	GNOME_VFS_XFER_PHASE_XFERRING,
+	/* Copying data from source to target (*) */
+	GNOME_VFS_XFER_PHASE_COPYING,
+	/* Moving file from source to target (*) */
+	GNOME_VFS_XFER_PHASE_MOVING,
 	/* Reading data from source file */
 	GNOME_VFS_XFER_PHASE_READSOURCE,
 	/* Writing data to target file */
@@ -490,7 +512,7 @@ enum _GnomeVFSXferPhase {
 	GNOME_VFS_XFER_PHASE_CLOSESOURCE,
 	/* Closing target file */
 	GNOME_VFS_XFER_PHASE_CLOSETARGET,
-	/* Deletin source file */
+	/* Deleting source file */
 	GNOME_VFS_XFER_PHASE_DELETESOURCE,
 	/* Setting attributes on target file */
 	GNOME_VFS_XFER_PHASE_SETATTRIBUTES,
@@ -500,11 +522,11 @@ enum _GnomeVFSXferPhase {
 	GNOME_VFS_XFER_PHASE_COMPLETED,
 	GNOME_VFS_XFER_NUM_PHASES
 };
-typedef enum _GnomeVFSXferPhase GnomeVFSXferPhase;
+typedef enum GnomeVFSXferPhase GnomeVFSXferPhase;
 
 /* Progress information for the transfer operation.  This is especially useful
    for interactive programs.  */
-struct _GnomeVFSXferProgressInfo {
+struct GnomeVFSXferProgressInfo {
 	/* Progress status (see above for a description).  */
 	GnomeVFSXferProgressStatus status;
 
@@ -516,10 +538,10 @@ struct _GnomeVFSXferProgressInfo {
 	/* Current phase in the transferring process.  */
 	GnomeVFSXferPhase phase;
 
-	/* Source URI.  FIXME name?  */
+	/* Source URI. FIXME bugzilla.eazel.com 1206: change name? */
 	gchar *source_name;
 
-	/* Destination URI.  FIXME name?  */
+	/* Destination URI. FIXME bugzilla.eazel.com 1206: change name? */
 	gchar *target_name;
 
 	/* Index of file being copied. */
@@ -539,8 +561,16 @@ struct _GnomeVFSXferProgressInfo {
 
 	/* Total amount of data copied from the beginning.  */
 	GnomeVFSFileSize total_bytes_copied;
+	
+	/* Target unique name used when duplicating, etc. to avoid collisions */ 
+	gchar *duplicate_name;
+
+	/* Count used in the unique name e.g. (copy 2), etc. */
+	int duplicate_count;
+
 };
-typedef struct _GnomeVFSXferProgressInfo GnomeVFSXferProgressInfo;
+
+typedef struct GnomeVFSXferProgressInfo GnomeVFSXferProgressInfo;
 
 /* This is the prototype for functions called during a transfer operation to
    report progress.  If the return value is `FALSE' (0), the operation is
@@ -559,13 +589,13 @@ typedef struct _GnomeVFSXferProgressInfo GnomeVFSXferProgressInfo;
    - If the status is `GNOME_VFS_XFER_PROGRESS_STATUS_OVERWRITE', the return
      value is interpreted as a `GnomeVFSXferOverwriteAction'.  */
 
-typedef gint (* GnomeVFSXferProgressCallback) 	(const GnomeVFSXferProgressInfo *info,
+typedef gint (* GnomeVFSXferProgressCallback) 	(GnomeVFSXferProgressInfo *info,
 						 gpointer data);
 
 
 /* Types for asynchronous operations.  */
 
-typedef struct _GnomeVFSAsyncHandle GnomeVFSAsyncHandle;
+typedef struct GnomeVFSAsyncHandle GnomeVFSAsyncHandle;
 
 typedef void	(* GnomeVFSAsyncOpenCallback)	(GnomeVFSAsyncHandle *handle,
 						 GnomeVFSResult result,
@@ -599,11 +629,16 @@ typedef void	(* GnomeVFSAsyncWriteCallback)	(GnomeVFSAsyncHandle *handle,
 						 GnomeVFSFileSize bytes_written,
 						 gpointer callback_data);
 
+typedef struct {
+	GnomeVFSURI *uri;
+	GnomeVFSResult result;
+	GnomeVFSFileInfo *file_info;
+} GnomeVFSGetFileInfoResult;
+
 typedef void    (* GnomeVFSAsyncGetFileInfoCallback)
-                                                 (GnomeVFSAsyncHandle *handle,
-						  GnomeVFSResult result,
-						  GnomeVFSFileInfo *file_info,
-						  gpointer callback_data);
+                                                (GnomeVFSAsyncHandle *handle,
+						 GList *results, /* GnomeVFSGetFileInfoResult* items */
+						 gpointer callback_data);
 
 typedef void	(* GnomeVFSAsyncDirectoryLoadCallback)
 						(GnomeVFSAsyncHandle *handle,
@@ -614,7 +649,7 @@ typedef void	(* GnomeVFSAsyncDirectoryLoadCallback)
 
 typedef gint    (* GnomeVFSAsyncXferProgressCallback)
 						(GnomeVFSAsyncHandle *handle,
-						 const GnomeVFSXferProgressInfo *info,
+						 GnomeVFSXferProgressInfo *info,
 						 gpointer data);
 
 
