@@ -23,8 +23,6 @@
    `split_toplevel_uri()' derived from Midnight Commander code by Norbert
    Warmuth, Miguel de Icaza, Janne Kukonlehto, Dugan Porter, Jakub Jelinek.  */
 
-/* TODO: %xx syntax for providing any character in the URI.  */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -116,30 +114,37 @@ split_toplevel_uri (const gchar *path, guint path_len,
 	}
 
 	/* Check if the host comes with a port spec, if so, chop it.  */
-	colon = memchr (rest, ':', dir - rest);
-	if (colon != NULL && colon != dir - 1) {
-		*host_return = g_strndup (rest, colon - rest);
+	if(dir) {
+		colon = memchr (rest, ':', dir - rest);
+		if (colon != NULL && colon != dir - 1) {
+			*host_return = g_strndup (rest, colon - rest);
 
-		if (sscanf (colon + 1, "%d", port_return) == 1) {
-			if (*port_return > 0xffff)
-				*port_return = 0;
-		} else {
-			while (1) {
-				colon++;
-				switch(*colon) {
-				case 'C':
-					*port_return = 1;
-					break;
-				case 'r':
-					*port_return = 2;
-					break;
-				case 0:
-					goto done;
+			if (sscanf (colon + 1, "%d", port_return) == 1) {
+				if (*port_return > 0xffff)
+					*port_return = 0;
+			} else {
+				while (1) {
+					colon++;
+					switch(*colon) {
+					case 'C':
+						*port_return = 1;
+						g_warning("Setting *port_return = 1");
+						break;
+					case 'r':
+						*port_return = 2;
+						g_warning("Setting *port_return = 2");
+						break;
+					case 0:
+						*port_return = 0;
+						goto done;
+					}
 				}
 			}
+		} else {
+			*host_return = g_strndup (rest, dir - rest);
 		}
 	} else {
-		*host_return = g_strndup (rest, dir - rest);
+		*host_return = g_strdup(path);
 	}
 
  done:
@@ -185,6 +190,64 @@ set_uri_element (GnomeVFSURI *uri,
 	gnome_vfs_canonicalize_pathname (uri->text);
 }
 
+static const gchar *
+get_method_string (const gchar *substring, gchar **method_string)
+{
+	const gchar *p;
+	
+	for (p = substring;
+	     isalnum ((unsigned char)*p) || *p == '+' || *p == '-' || *p == '.';
+	     p++)
+		;
+
+	if (*p == ':') {
+		/* Found toplevel method specification.  */
+		*method_string = g_strndup (substring, p - substring);
+		p++;
+	} else {
+		*method_string = g_strdup ("file");
+		p = substring;
+	}
+	return p;
+}
+
+static GnomeVFSURI *
+parse_uri_substring (const gchar *substring, GnomeVFSURI *parent)
+{
+	GnomeVFSMethod *method;
+	GnomeVFSURI    *uri, *new_uri;
+	gchar          *method_string;
+	const gchar    *p;
+	gchar          *p1;
+
+	if (!substring || *substring == '\000')
+		return NULL;
+
+	p = get_method_string (substring, &method_string);
+
+	method = gnome_vfs_method_get (method_string);
+	if (!method) {
+		g_free (method_string);
+		return NULL;
+	}
+
+	uri = g_new0 (GnomeVFSURI, 1);
+	uri->method        = method;
+	uri->method_string = method_string;
+	uri->ref_count     = 1;
+	uri->parent        = parent;
+
+	p1 = strchr (p, GNOME_VFS_URI_MAGIC_CHR);
+	if (!p1) {
+		set_uri_element (uri, p, strlen (p));
+		return uri;
+	}
+
+	set_uri_element (uri, p, p1 - p);
+	new_uri = parse_uri_substring (p1 + 1, uri);
+
+	return uri;
+}
 /**
  * gnome_vfs_uri_new:
  * @text_uri: A string representing a URI.
@@ -197,95 +260,80 @@ GnomeVFSURI *
 gnome_vfs_uri_new (const gchar *text_uri)
 {
 	GnomeVFSMethod *method;
+	GnomeVFSTransform *trans;
 	GnomeVFSToplevelURI *toplevel;
 	GnomeVFSURI *uri, *new_uri;
-	const gchar *p;
+	const gchar *p, *p1, *p2;
 	gchar *method_string;
+	gchar *new_uri_string = NULL;
 
 	g_return_val_if_fail (text_uri != NULL, NULL);
 
 	if (text_uri[0] == 0)
 		return NULL;
 
-	/* FIXME: Correct to look for alpha only?  */
-	for (p = text_uri; isalpha (*p); p++)
-		;
-
-	if (*p == ':') {
-		/* Found toplevel method specification.  */
-		ALLOCA_SUBSTRING (method_string, text_uri, p - text_uri);
-		method = gnome_vfs_method_get (method_string);
-		if (method == NULL)
-			return NULL;
-		p++;
-	} else {
-		/* No toplevel method specification.  Use "file" as
-                   the default.  */
-		method_string = "file";
-		method = gnome_vfs_method_get (method_string);
-		if (method == NULL)
-			return NULL;
-		p = text_uri;
-	}
-
-	/* The toplevel URI element is special, as it also contains host/user
-           information.  */
 	toplevel = g_new (GnomeVFSToplevelURI, 1);
 	toplevel->host_name = NULL;
 	toplevel->host_port = 0;
 	toplevel->user_name = NULL;
 	toplevel->password = NULL;
 
-	new_uri = (GnomeVFSURI *) toplevel;
-	new_uri->parent = NULL;
+	uri = (GnomeVFSURI *) toplevel;
+	uri->parent = NULL;
 
-	while (1) {
-		GnomeVFSMethod *new_method;
-		const gchar *p1, *p2;
-		gchar *new_method_string;
+	p = get_method_string (text_uri, &method_string);
+	trans = gnome_vfs_transform_get (method_string);
+	if (trans && trans->transform) {
+		GnomeVFSContext *context;
 
-		new_uri->method = method;
-		new_uri->method_string = g_strdup (method_string);
-		new_uri->ref_count = 1;
-
-		p1 = strchr (p, GNOME_VFS_URI_MAGIC_CHR);
-		if (p1 == NULL) {
-			set_uri_element (new_uri, p, strlen (p));
-			uri = new_uri;
-			break;
+		context = gnome_vfs_context_new ();
+		(* trans->transform) (trans, p, &new_uri_string, context);
+		g_print ("new_uri_string: %s\n", new_uri_string);
+		gnome_vfs_context_unref (context);
+		if (new_uri_string != NULL) {
+			toplevel->urn = g_strdup (text_uri);
+			g_free (method_string);
+			p = get_method_string (new_uri_string, &method_string);
 		}
-
-		set_uri_element (new_uri, p, p1 - p);
-
-		p2 = p1 + 1;
-		if (*p2 == 0) {
-			gnome_vfs_uri_unref (new_uri);
-			return NULL;
-		}
-
-		while (*p2 != 0 && *p2 != GNOME_VFS_URI_PATH_CHR && *p2 != GNOME_VFS_URI_MAGIC_CHR)
-			p2++;
-
-		ALLOCA_SUBSTRING (new_method_string, p1 + 1, p2 - p1 - 1);
-		new_method = gnome_vfs_method_get (new_method_string);
-
-		if (new_method == NULL) {
-			/* FIXME: Better error handling/reporting?  */
-			gnome_vfs_uri_unref (new_uri);
-			return NULL;
-		}
-
-		p = p2;
-
-		method = new_method;
-		method_string = new_method_string;
-		uri = new_uri;
-
-		new_uri = g_new (GnomeVFSURI, 1);
-		new_uri->parent = uri;
+	}
+	
+	method = gnome_vfs_method_get (method_string);
+	/* The toplevel URI element is special, as it also contains host/user
+           information.  */
+	uri->method        = method;
+	uri->ref_count     = 1;
+	uri->method_string = method_string;
+	uri->text          = NULL;
+	if (method == NULL) {
+		gnome_vfs_uri_unref (uri);
+		g_free (new_uri_string);
+		return NULL;
+	}
+	
+	p1 = strchr (p, GNOME_VFS_URI_MAGIC_CHR);
+	if (p1 == NULL) {
+		set_uri_element (uri, p, strlen (p));
+		g_free (new_uri_string);
+		return uri;
 	}
 
-	return uri;
+	set_uri_element (uri, p, p1 - p);
+
+	p2 = p1 + 1;
+		
+	if (*p2 == 0) {
+		gnome_vfs_uri_unref (uri);
+		g_free (new_uri_string);
+		return NULL;
+	}
+	new_uri = parse_uri_substring (p2, uri);
+
+	g_free (new_uri_string);
+
+	if (new_uri)
+		return new_uri;
+	else
+		return uri;
 }
 
 
@@ -412,9 +460,9 @@ gnome_vfs_uri_dup (const GnomeVFSURI *uri)
 
 
 /**
- * gnome_vfs_uri_append_path:
+ * gnome_vfs_uri_append_append_path:
  * @uri: A GnomeVFSURI.
- * @path: A path component.
+ * @path: A piece of a URI (ie a fully escaped partial path)
  * 
  * Create a new URI obtained by appending @path to @uri.  This will take care
  * of adding an appropriate directory separator between the end of @uri and
@@ -422,8 +470,6 @@ gnome_vfs_uri_dup (const GnomeVFSURI *uri)
  * 
  * Return value: The new URI obtained by combining @uri and @path.
  **/
-/* FIXME this must be implemented in a much smarter way.  The most important
-   issue is that a `#' in `path' will break things rather badly.  */
 GnomeVFSURI *
 gnome_vfs_uri_append_path (const GnomeVFSURI *uri,
 			   const gchar *path)
@@ -436,7 +482,7 @@ gnome_vfs_uri_append_path (const GnomeVFSURI *uri,
 	g_return_val_if_fail (uri != NULL, NULL);
 	g_return_val_if_fail (path != NULL, NULL);
 
-	/* FIXME this is just a reminder.  */
+	/* FIXME bugzilla.eazel.com 1209: this is just a reminder.  */
 	if (strchr (path, '#') != NULL)
 		g_warning ("gnome_vfs_uri_append_path() is broken with names containing `#'.");
 
@@ -464,6 +510,30 @@ gnome_vfs_uri_append_path (const GnomeVFSURI *uri,
 	return new;
 }
 
+/**
+ * gnome_vfs_uri_append_file_name:
+ * @uri: A GnomeVFSURI.
+ * @path: any "regular" file name (can include #, /, etc)
+ * 
+ * Create a new URI obtained by appending @file_name to @uri.  This will take care
+ * of adding an appropriate directory separator between the end of @uri and
+ * the start of @file_name if necessary.
+ * 
+ * Return value: The new URI obtained by combining @uri and @path.
+ **/
+GnomeVFSURI *
+gnome_vfs_uri_append_file_name (const GnomeVFSURI *uri,
+				const gchar *file_name)
+{
+	gchar *escaped_string;
+	GnomeVFSURI *new_uri;
+	
+	escaped_string = gnome_vfs_escape_string (file_name);
+	new_uri = gnome_vfs_uri_append_path (uri, escaped_string);
+	g_free (escaped_string);
+	return new_uri;
+}
+
 
 /**
  * gnome_vfs_uri_to_string:
@@ -486,9 +556,6 @@ gnome_vfs_uri_to_string (const GnomeVFSURI *uri,
 	guint size, len;
 	gchar *s, *p;
 
-	/* FIXME/TODO: Special characters such as `#' in the URI components
-           should be replaced.  */
-
 	g_return_val_if_fail (uri != NULL, NULL);
 
 	size = 0;
@@ -606,12 +673,15 @@ gnome_vfs_uri_is_local (const GnomeVFSURI *uri)
 {
 	g_return_val_if_fail (uri != NULL, FALSE);
 
+	/* It's illegal to have is_local be NULL in a method.
+	 * That's why we fail here. If we decide that it's legal,
+	 * then we can change this into an if statement.
+	 */
+	g_return_val_if_fail (uri->method->is_local != NULL, FALSE);
+
 	return uri->method->is_local (uri->method, uri);
 }
 
-/* FIXME Maybe we have to make more definite decisions on what URLs have to
-   look like here.  */
-
 /**
  * gnome_vfs_uri_has_parent:
  * @uri: A GnomeVFSURI.
@@ -657,34 +727,39 @@ gnome_vfs_uri_get_parent (const GnomeVFSURI *uri)
 		len = strlen (uri->text);
 		p = uri->text + len - 1;
 
-		/* Skip trailing slash.  */
-		if (*p == GNOME_VFS_URI_PATH_CHR && p != uri->text)
+		/* Skip trailing slashes  */
+		while (p != uri->text && *p == GNOME_VFS_URI_PATH_CHR)
 			p--;
 
-		/* Search backwards for next slash.  URIs are normalized, so
-                   the next slash we find cannot be a trailing one.  */
+		/* Search backwards to the next slash.  */
 		while (p != uri->text && *p != GNOME_VFS_URI_PATH_CHR)
 			p--;
 
+		/* Get the parent without slashes  */
+		while (p > uri->text + 1 && p[-1] == GNOME_VFS_URI_PATH_CHR)
+			p--;
+
 		if (p[1] != '\0') {
-			GnomeVFSURI *new;
+			GnomeVFSURI *new_uri;
+			char *new_uri_text;
+			int length;
 
-			/* The following is not extremely efficient, but quite
-			   safe in case we change the internals.  */
-
-			new = gnome_vfs_uri_dup (uri);
-			g_free (new->text);
-
-			if (p == uri->text) {
-				new->text = g_strdup (GNOME_VFS_URI_PATH_STR);
+			/* build a new parent text */
+			length = p - uri->text;			
+			if (length == 0) {
+				new_uri_text = g_strdup (GNOME_VFS_URI_PATH_STR);
 			} else {
-				new->text = g_malloc (p - uri->text + 2);
-				memcpy (new->text, uri->text, p - uri->text);
-				new->text[p - uri->text] = GNOME_VFS_URI_PATH_CHR;
-				new->text[p - uri->text + 1] = '\0';
+				new_uri_text = g_malloc (length + 1);
+				memcpy (new_uri_text, uri->text, length);
+				new_uri_text[length] = '\0';
 			}
 
-			return new;
+			/* copy the uri and replace the uri text with the new parent text */
+			new_uri = gnome_vfs_uri_dup (uri);
+			g_free (new_uri->text);
+			new_uri->text = new_uri_text;
+			
+			return new_uri;
 		}
 	}
 
@@ -733,6 +808,20 @@ gnome_vfs_uri_get_host_name (const GnomeVFSURI *uri)
 	return toplevel->host_name;
 }
 
+/**
+ * gnome_vfs_uri_get_scheme:
+ * @uri: A GnomeVFSURI
+ *
+ * Retrieve the scheme used for @uri
+ *
+ * Return value: A string representing the scheme
+ **/
+const gchar *
+gnome_vfs_uri_get_scheme (const GnomeVFSURI *uri)
+{
+	return uri->method_string;
+}
+
 /**
  * gnome_vfs_uri_get_host_port:
  * @uri: A GnomeVFSURI.
@@ -879,20 +968,17 @@ gnome_vfs_uri_set_password (GnomeVFSURI *uri,
 	toplevel->host_name = g_strdup (password);
 }
 
-
-static gint
+static gboolean
 my_streq (const gchar *a,
 	  const gchar *b)
 {
-	if (a == NULL) {
-		if (b == NULL)
-			return TRUE;
-		else
-			return FALSE;
-	} else if (b == NULL) {
-		return FALSE;
+	if (a == NULL || *a == '\0') {
+		return b == NULL || *b == '\0';
 	}
 
+	if (a == NULL || b == NULL)
+		return FALSE;
+
 	return strcmp (a, b) == 0;
 }
 
@@ -956,7 +1042,81 @@ gnome_vfs_uri_equal (const GnomeVFSURI *a,
 	return TRUE;
 }
 
+/**
+ * gnome_vfs_uri_is_parent:
+ * @possible_parent: A GnomeVFSURI.
+ * @possible_child: A GnomeVFSURI.
+ * @recursive: a flag to turn recursive check on.
+ * 
+ * Check if @possible_child is contained by @possible_parent.
+ * If @recursive is FALSE, just try the immediate parent directory, else
+ * search up through the hierarchy.
+ * 
+ * Return value: %TRUE if @possible_child is contained in  @possible_child.
+ **/
+gboolean
+gnome_vfs_uri_is_parent (const GnomeVFSURI *possible_parent,
+			 const GnomeVFSURI *possible_child,
+			 gboolean recursive)
+{
+	gboolean result;
+	GnomeVFSURI *item_parent_uri;
+	GnomeVFSURI *item;
+
+	if (!recursive) {
+		item_parent_uri = gnome_vfs_uri_get_parent (possible_child);
+
+		if (item_parent_uri == NULL) 
+			return FALSE;
+
+		result = gnome_vfs_uri_equal (item_parent_uri, possible_parent);	
+		gnome_vfs_uri_unref (item_parent_uri);
+
+		return result;
+	}
+	
+	item = gnome_vfs_uri_dup (possible_child);
+	for (;;) {
+		item_parent_uri = gnome_vfs_uri_get_parent (item);
+		gnome_vfs_uri_unref (item);
+		
+		if (item_parent_uri == NULL) 
+			return FALSE;
+
+		result = gnome_vfs_uri_equal (item_parent_uri, possible_parent);
+	
+		if (result) {
+			gnome_vfs_uri_unref (item_parent_uri);
+			break;
+		}
+
+		item = item_parent_uri;
+	}
+
+	return result;
+}
+
 
+/**
+ * gnome_vfs_uri_get_path:
+ * @uri: A GnomeVFSURI
+ * 
+ * Retrieve full path name for @uri.
+ * 
+ * Return value: A pointer to the full path name in @uri.  Notice that the
+ * pointer points to the name store in @uri, so the name returned must not
+ * be modified nor freed.
+ **/
+const gchar *
+gnome_vfs_uri_get_path (const GnomeVFSURI *uri)
+{
+	/* FIXME bugzilla.eazel.com 1472 */
+	/* this is based on the assumtion that uri->text won't contain the
+	 * query string.
+	 */
+	return uri->text;
+}
+
 /**
  * gnome_vfs_uri_get_basename:
  * @uri: A GnomeVFSURI
@@ -970,10 +1130,14 @@ gnome_vfs_uri_equal (const GnomeVFSURI *a,
 const gchar *
 gnome_vfs_uri_get_basename (const GnomeVFSURI *uri)
 {
+	/* FIXME bugzilla.eazel.com 1472: query parts of URIs aren't handled */
 	gchar *p;
 
 	g_return_val_if_fail (uri != NULL, NULL);
 
+	if (uri->text == NULL)
+		return NULL;
+
 	p = strrchr (uri->text, GNOME_VFS_URI_PATH_CHR);
 	if (p == NULL)
 		return NULL;
@@ -1018,19 +1182,63 @@ gnome_vfs_uri_extract_dirname (const GnomeVFSURI *uri)
  * This matches the XPG definition of basename, but not g_basename. This is
  * often useful when you want the name of something that's pointed to by a
  * uri, and don't care whether the uri has a directory or file form.
- * If @uri points to the root, returns GNOME_VFS_URI_PATH_STR.
+ * If @uri points to the root of a domain, returns the host name. If there's
+ * no host name, returns GNOME_VFS_URI_PATH_STR.
+ * 
+ * See also: gnome_vfs_uri_extract_short_path_name.
  * 
  * Return value: A pointer to the newly allocated string representing the
- * short form of the name.
+ * unescaped short form of the name.
  **/
 gchar *
 gnome_vfs_uri_extract_short_name (const GnomeVFSURI *uri)
+{
+	gchar *escaped_short_path_name, *short_path_name;
+	const gchar *host_name;
+
+	escaped_short_path_name = gnome_vfs_uri_extract_short_path_name (uri);
+	short_path_name = gnome_vfs_unescape_string_for_display (escaped_short_path_name);
+	g_free (escaped_short_path_name);
+
+	host_name = NULL;
+	if (short_path_name != NULL
+	    && strcmp (short_path_name, GNOME_VFS_URI_PATH_STR) == 0) {
+		host_name = gnome_vfs_uri_get_host_name (uri);
+	}
+
+	if (host_name == NULL || strlen (host_name) == 0) {
+		return short_path_name;
+	}
+
+	g_free (short_path_name);
+	return g_strdup (host_name);
+}
+
+/**
+ * gnome_vfs_uri_extract_short_path_name:
+ * @uri: A GnomeVFSURI
+ * 
+ * Retrieve base file name for @uri, ignoring any trailing path separators.
+ * This matches the XPG definition of basename, but not g_basename. This is
+ * often useful when you want the name of something that's pointed to by a
+ * uri, and don't care whether the uri has a directory or file form.
+ * If @uri points to the root (including the root of any domain),
+ * returns GNOME_VFS_URI_PATH_STR.
+ * 
+ * See also: gnome_vfs_uri_extract_short_name.
+ * 
+ * Return value: A pointer to the newly allocated string representing the
+ * escaped short form of the name.
+ **/
+gchar *
+gnome_vfs_uri_extract_short_path_name (const GnomeVFSURI *uri)
 {
 	const gchar *p, *short_name_start, *short_name_end;
 
 	g_return_val_if_fail (uri != NULL, NULL);
-	g_return_val_if_fail (uri->text != NULL, NULL);
-	g_return_val_if_fail (uri->text[0] != '\0', NULL);
+
+	if (uri->text == NULL)
+		return NULL;
 
 	/* Search for the last run of non-'/' characters. */
 	p = uri->text;
@@ -1110,3 +1318,29 @@ gnome_vfs_uri_hash (gconstpointer p)
 #undef HASH_STRING
 #undef HASH_NUMBER
 }
+
+GList *
+gnome_vfs_uri_list_ref (GList *list)
+{
+	g_list_foreach (list, (GFunc) gnome_vfs_uri_ref, NULL);
+	return list;
+}
+
+GList *
+gnome_vfs_uri_list_unref (GList *list)
+{
+	g_list_foreach (list, (GFunc) gnome_vfs_uri_unref, NULL);
+	return list;
+}
+
+GList *
+gnome_vfs_uri_list_copy (GList *list)
+{
+	return g_list_copy (gnome_vfs_uri_list_ref (list));
+}
+
+void
+gnome_vfs_uri_list_free (GList *list)
+{
+	g_list_free (gnome_vfs_uri_list_unref (list));
+}
