@@ -43,9 +43,10 @@
 
 #include "gnome-vfs.h"
 #include "gnome-vfs-private.h"
+#include "gnome-vfs-mime.h"
 
 #include "gnome-vfs-module.h"
-#include "module-shared.h"
+#include "gnome-vfs-module-shared.h"
 
 #include "extfs-method.h"
 
@@ -96,7 +97,7 @@ struct _ExtfsDirectoryHandle {
 	ExtfsDirectory *directory;
 	GList *prev_position;
 	gchar *sub_uri;
-	gchar **meta_keys;
+	GList *meta_keys;
 	GnomeVFSFileInfoOptions info_options;
 	const GnomeVFSDirectoryFilter *filter;
 };
@@ -104,8 +105,8 @@ typedef struct _ExtfsDirectoryHandle ExtfsDirectoryHandle;
 
 
 #define ERROR_IF_NOT_LOCAL(uri)					\
-	if (strcmp ((uri)->parent->method_string, "file") != 0)	\
-		return GNOME_VFS_ERROR_NOTSUPPORTED;
+	if ((!uri) || (!uri->parent) || (!(uri)->parent->method_string) || strcmp ((uri)->parent->method_string, "file") != 0)	\
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 
 static GnomeVFSResult
 extfs_handle_close (ExtfsHandle *handle)
@@ -208,7 +209,7 @@ free_directory_entries (GList *entries)
 		ExtfsDirectoryEntry *entry;
 
 		entry = p->data;
-		gnome_vfs_file_info_destroy (entry->info);
+		gnome_vfs_file_info_unref (entry->info);
 		g_free (entry->directory);
 		g_free (entry);
 	}
@@ -294,7 +295,7 @@ do_open (GnomeVFSMethod *method,
 	ExtfsHandle *handle;
 	gchar *script_path;
 	const gchar *stored_name;
-	gchar *args[6];
+	const gchar *args[6];
 	gchar *temp_name;
 	gboolean cleanup;
 	gint process_exit_value;
@@ -303,10 +304,10 @@ do_open (GnomeVFSMethod *method,
 
 	/* TODO: Support write mode.  */
 	if (mode & GNOME_VFS_OPEN_WRITE)
-		return GNOME_VFS_ERROR_READONLYFS;
+		return GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM;
 
 	if (uri->text == NULL)
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	if (uri->method_string == NULL)
 		return GNOME_VFS_ERROR_INTERNAL;
@@ -316,7 +317,7 @@ do_open (GnomeVFSMethod *method,
 		stored_name++;
 
 	if (*stored_name == '\0')
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	result = gnome_vfs_create_temp ("extfs", &temp_name, &temp_handle);
 	if (result != GNOME_VFS_OK)
@@ -336,7 +337,11 @@ do_open (GnomeVFSMethod *method,
 	args[4] = temp_name;
 	args[5] = NULL;
 	
-	/* FIXME args */
+	/* FIXME bugzilla.eazel.com 1223:
+	 * args 
+	 * Ettore needs to elaborate here some more, it is not clear what this
+	 * FixMe is about
+	 */
 	process_result = gnome_vfs_process_run_cancellable
 		(script_path, args, GNOME_VFS_PROCESS_CLOSEFDS,
 		 context ? gnome_vfs_context_get_cancellation(context) : NULL,
@@ -350,7 +355,7 @@ do_open (GnomeVFSMethod *method,
 		} else {
 			/* This is not very accurate, but it should be
 			   enough.  */
-			result = GNOME_VFS_ERROR_NOTFOUND;
+			result = GNOME_VFS_ERROR_NOT_FOUND;
 			cleanup = TRUE;
 		}
 		break;
@@ -397,7 +402,7 @@ do_create (GnomeVFSMethod *method,
 	   guint perm,
 	   GnomeVFSContext *context)
 {
-	return GNOME_VFS_ERROR_READONLYFS;
+	return GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM;
 }
 
 static GnomeVFSResult
@@ -441,7 +446,7 @@ do_write (GnomeVFSMethod *method,
 	  GnomeVFSFileSize *bytes_written,
 	  GnomeVFSContext *context)
 {
-	return GNOME_VFS_ERROR_READONLYFS;
+	return GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM;
 }
 
 static GnomeVFSResult
@@ -470,7 +475,7 @@ do_truncate_handle (GnomeVFSMethod *method,
 		    GnomeVFSFileSize where,
 		    GnomeVFSContext *context)
 {
-	return GNOME_VFS_ERROR_NOTSUPPORTED;
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
 static GnomeVFSResult
@@ -479,7 +484,7 @@ do_truncate (GnomeVFSMethod *method,
 	     GnomeVFSFileSize where,
 	     GnomeVFSContext *context)
 {
-	return GNOME_VFS_ERROR_NOTSUPPORTED;
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
 
@@ -517,7 +522,7 @@ read_directory_list (FILE *p,
 		if (chars_read == -1)
 			break;
 
-		/* FIXME */
+		/* FIXME bugzilla.eazel.com 1223: */
 		fputs (line_buffer, stdout);
 
 		line_buffer[chars_read] = '\0';
@@ -537,8 +542,8 @@ read_directory_list (FILE *p,
 
 		/* Notice that we always do stupid, fast MIME type checking.
                    Real checking based on contents would be too expensive.  */
-		if (info_options & GNOME_VFS_FILE_INFO_GETMIMETYPE) {
-			info->mime_type = g_strdup (gnome_mime_type
+		if (info_options & GNOME_VFS_FILE_INFO_GET_MIME_TYPE) {
+			info->mime_type = g_strdup (gnome_vfs_mime_type_from_name
 						    (info->name));
 			info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
 		}
@@ -573,6 +578,7 @@ do_open_directory (GnomeVFSMethod *method,
 	gchar *quoted_file_name;
 	gchar *cmd;
 	const gchar *p;
+	const GList *item;
 	FILE *pipe;
 
 	ERROR_IF_NOT_LOCAL (uri);
@@ -584,7 +590,7 @@ do_open_directory (GnomeVFSMethod *method,
 
 		/* Check that the file exists first.  */
 		if (stat (uri->parent->text, &statbuf) != 0)
-			return GNOME_VFS_ERROR_NOTFOUND;
+			return GNOME_VFS_ERROR_NOT_FOUND;
 
 		quoted_file_name = quote_file_name (uri->parent->text);
 		script_path = get_script_path (uri);
@@ -598,7 +604,7 @@ do_open_directory (GnomeVFSMethod *method,
 		g_free (quoted_file_name);
 
 		if (pipe == NULL)
-			return GNOME_VFS_ERROR_NOTSUPPORTED;
+			return GNOME_VFS_ERROR_NOT_SUPPORTED;
 
 		result = read_directory_list (pipe, &list, info_options,
 					      context);
@@ -608,7 +614,7 @@ do_open_directory (GnomeVFSMethod *method,
 		} else {
 			free_directory_entries (list);
 			if (result == GNOME_VFS_OK)
-				return GNOME_VFS_ERROR_IO; /* FIXME? */
+				return GNOME_VFS_ERROR_IO; /* FIXME bugzilla.eazel.com 1223:? */
 			else
 				return result;
 		}
@@ -617,9 +623,12 @@ do_open_directory (GnomeVFSMethod *method,
 	handle = g_new (ExtfsDirectoryHandle, 1);
 	handle->directory = directory;
 	handle->prev_position = NULL;
-#warning "This meta_keys needs duplicating"
-	handle->meta_keys = meta_keys; /* FIXME currently unused FIXME strdup? */
-	handle->info_options = info_options; /* FIXME currently unused */
+	handle->meta_keys = NULL; /* FIXME bugzilla.eazel.com 1120: currently unused */
+	for (item = meta_keys; item != NULL; item = item->next) {
+		handle->meta_keys = g_list_prepend (handle->meta_keys, g_strdup (item->data));
+	}
+
+	handle->info_options = info_options; /* FIXME bugzilla.eazel.com 1224: currently unused */
 	handle->filter = filter;
 
 	/* Remove all leading slashes, as they don't matter for us.  */
@@ -642,13 +651,19 @@ do_close_directory (GnomeVFSMethod *method,
 		    GnomeVFSContext *context)
 {
 	ExtfsDirectoryHandle *handle;
+	GList *item;
 
 	handle = (ExtfsDirectoryHandle *) method_handle;
 
 	extfs_directory_unref (handle->directory);
 	g_free (handle->sub_uri);
+	for (item = handle->meta_keys; item != NULL; item = item->next) {
+		g_free (item->data);
+	}
+	g_list_free (handle->meta_keys);
 	g_free (handle);
 
+
 	return GNOME_VFS_OK;
 }
 
@@ -664,7 +679,8 @@ match (const ExtfsDirectoryHandle *handle,
 		return FALSE;
 
 	/* First check that this is in the subdirectory we want.  */
-	/* FIXME more canonicalization might be needed.  */
+	/* FIXME bugzilla.eazel.com 1125: 
+	 * more canonicalization might be needed.  */
 
 	if (entry->directory != NULL) {
 		for (p = entry->directory; *p == G_DIR_SEPARATOR; p++)
@@ -743,7 +759,7 @@ do_get_file_info (GnomeVFSMethod *method,
 		  const GList *meta_keys,
 		  GnomeVFSContext *context)
 {
-	return GNOME_VFS_ERROR_NOTSUPPORTED;
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
 static GnomeVFSResult
@@ -754,7 +770,7 @@ do_get_file_info_from_handle (GnomeVFSMethod *method,
 			      const GList *meta_keys,
 			      GnomeVFSContext *context)
 {
-	return GNOME_VFS_ERROR_NOTSUPPORTED;
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
 static gboolean
@@ -770,15 +786,28 @@ do_make_directory (GnomeVFSMethod *method,
 		   guint perm,
 		   GnomeVFSContext *context)
 {
-	return GNOME_VFS_ERROR_NOTSUPPORTED;
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
+static GnomeVFSResult
+do_find_directory (GnomeVFSMethod *method,
+		   GnomeVFSURI *near_uri,
+		   GnomeVFSFindDirectoryKind kind,
+		   GnomeVFSURI **result_uri,
+		   gboolean create_if_needed,
+		   guint permissions,
+		   GnomeVFSContext *context)
+{
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
+}
+
+
 static GnomeVFSResult
 do_remove_directory (GnomeVFSMethod *method,
 		     GnomeVFSURI *uri,
 		     GnomeVFSContext *context)
 {
-	return GNOME_VFS_ERROR_NOTSUPPORTED;
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
 static GnomeVFSResult
@@ -788,7 +817,7 @@ do_move (GnomeVFSMethod *method,
 	 gboolean force_replace,
 	 GnomeVFSContext *context)
 {
-	return GNOME_VFS_ERROR_NOTSUPPORTED;
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
 
@@ -797,7 +826,7 @@ do_unlink (GnomeVFSMethod *method,
 	   GnomeVFSURI *uri,
 	   GnomeVFSContext *context)
 {
-	return GNOME_VFS_ERROR_NOTSUPPORTED;
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
 static GnomeVFSResult
@@ -807,7 +836,7 @@ do_check_same_fs (GnomeVFSMethod *method,
 		  gboolean *same_fs_return,
 		  GnomeVFSContext *context)
 {
-	return GNOME_VFS_ERROR_NOTSUPPORTED;
+	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 }
 
 
@@ -832,7 +861,9 @@ static GnomeVFSMethod method = {
 	do_unlink,
 	do_check_same_fs,
 	NULL,
-	do_truncate
+	do_truncate,
+	do_find_directory,
+	NULL     /* FIXME: do_create_symbolic_link */
 };
 
 GnomeVFSMethod *
