@@ -21,8 +21,6 @@
 
    Author: Ettore Perazzoli <ettore@comm2000.it> */
 
-/* TODO metadata? */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -40,23 +38,12 @@
 
 #include <gnome.h>
 
+#include "gnome-vfs-mime.h"
+
 #include "gnome-vfs-module.h"
-#include "module-shared.h"
+#include "gnome-vfs-module-shared.h"
 #include "file-method.h"
 
-
-/* This is to make sure the path starts with `/', so that at least we
-   get a predictable behavior when the leading `/' is not present.  */
-#define MAKE_ABSOLUTE(dest, src)			\
-G_STMT_START{						\
-	if ((src)[0] != '/') {				\
-		(dest) = alloca (strlen (src) + 2);	\
-		(dest)[0] = '/';			\
-		strcpy ((dest), (src));			\
-	} else {					\
-		(dest) = (src);				\
-	}						\
-}G_STMT_END
 
 #ifdef PATH_MAX
 #define	GET_PATH_MAX()	PATH_MAX
@@ -96,25 +83,54 @@ GET_PATH_MAX (void)
 #define OFF_T off_t
 #endif
 
+static gchar *
+get_path_from_uri (GnomeVFSURI *uri)
+{
+	gchar *path, *longer_path;
+
+	path = gnome_vfs_unescape_string (uri->text, "/");
+	if (path == NULL)
+		return NULL;
+
+	/* This is to make sure the path starts with a "/", so that at
+	 * least we get a predictable behavior when the
+	 * leading "/" is not present.
+	 */
+	if (path[0] == '/')
+		return path;
+	longer_path = g_strconcat ("/", path, NULL);
+	g_free (path);
+	return longer_path;
+}
+
+static gchar *
+get_base_from_uri (GnomeVFSURI *uri)
+{
+	gchar *escaped_base, *base;
+
+	escaped_base = gnome_vfs_uri_extract_short_path_name (uri);
+	base = gnome_vfs_unescape_string (escaped_base, "/");
+	g_free (escaped_base);
+	return base;
+}
+
 
-struct _FileHandle {
+typedef struct {
 	GnomeVFSURI *uri;
 	gint fd;
-};
-typedef struct _FileHandle FileHandle;
+} FileHandle;
 
 static FileHandle *
 file_handle_new (GnomeVFSURI *uri,
 		 gint fd)
 {
-	FileHandle *new;
+	FileHandle *result;
+	result = g_new (FileHandle, 1);
 
-	new = g_new (FileHandle, 1);
+	result->uri = gnome_vfs_uri_ref (uri);
+	result->fd = fd;
 
-	new->uri = gnome_vfs_uri_ref (uri);
-	new->fd = fd;
-
-	return new;
+	return result;
 }
 
 static void
@@ -150,19 +166,23 @@ do_open (GnomeVFSMethod *method,
 		if (mode & GNOME_VFS_OPEN_WRITE)
 			unix_mode = O_WRONLY;
 		else
-			return GNOME_VFS_ERROR_INVALIDOPENMODE;
+			return GNOME_VFS_ERROR_INVALID_OPEN_MODE;
 	}
 
 	if (! (mode & GNOME_VFS_OPEN_RANDOM) && (mode & GNOME_VFS_OPEN_WRITE))
 		mode |= O_TRUNC;
 
-	MAKE_ABSOLUTE (file_name, uri->text);
+	file_name = get_path_from_uri (uri);
+	if (file_name == NULL)
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	do
 		fd = OPEN (file_name, unix_mode);
 	while (fd == -1
 	       && errno == EINTR
-	       && ! gnome_vfs_context_check_cancellation(context));
+	       && ! gnome_vfs_context_check_cancellation (context));
+
+	g_free (file_name);
 
 	if (fd == -1)
 		return gnome_vfs_result_from_errno ();
@@ -172,7 +192,7 @@ do_open (GnomeVFSMethod *method,
 
 	if (S_ISDIR (statbuf.st_mode)) {
 		close (fd);
-		return GNOME_VFS_ERROR_ISDIRECTORY;
+		return GNOME_VFS_ERROR_IS_DIRECTORY;
 	}
 
 	file_handle = file_handle_new (uri, fd);
@@ -202,7 +222,7 @@ do_create (GnomeVFSMethod *method,
 	unix_mode = O_CREAT | O_TRUNC;
 	
 	if (!(mode & GNOME_VFS_OPEN_WRITE))
-		return GNOME_VFS_ERROR_INVALIDOPENMODE;
+		return GNOME_VFS_ERROR_INVALID_OPEN_MODE;
 
 	if (mode & GNOME_VFS_OPEN_READ)
 		unix_mode |= O_RDWR;
@@ -212,7 +232,9 @@ do_create (GnomeVFSMethod *method,
 	if (exclusive)
 		unix_mode |= O_EXCL;
 
-	MAKE_ABSOLUTE (file_name, uri->text);
+	file_name = get_path_from_uri (uri);
+	if (file_name == NULL)
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	do
 		fd = OPEN (file_name, unix_mode, perm);
@@ -220,6 +242,8 @@ do_create (GnomeVFSMethod *method,
 	       && errno == EINTR
 	       && ! gnome_vfs_context_check_cancellation (context));
 
+	g_free (file_name);
+
 	if (fd == -1)
 		return gnome_vfs_result_from_errno ();
 
@@ -246,15 +270,16 @@ do_close (GnomeVFSMethod *method,
 		close_retval = close (file_handle->fd);
 	while (close_retval != 0
 	       && errno == EINTR
-	       && ! gnome_vfs_context_check_cancellation(context));
+	       && ! gnome_vfs_context_check_cancellation (context));
 
-	/* FIXME: Should do this even after a failure?  */
+	/* FIXME bugzilla.eazel.com 1163: Should do this even after a failure?  */
 	file_handle_destroy (file_handle);
 
-	if (close_retval == 0)
-		return GNOME_VFS_OK;
-	else
+	if (close_retval != 0) {
 		return gnome_vfs_result_from_errno ();
+	}
+
+	return GNOME_VFS_OK;
 }
 
 static GnomeVFSResult
@@ -276,7 +301,7 @@ do_read (GnomeVFSMethod *method,
 		read_val = read (file_handle->fd, buffer, num_bytes);
 	while (read_val == -1
 	       && errno == EINTR
-	       && ! gnome_vfs_context_check_cancellation(context));
+	       && ! gnome_vfs_context_check_cancellation (context));
 
 	if (read_val == -1) {
 		*bytes_read = 0;
@@ -306,7 +331,7 @@ do_write (GnomeVFSMethod *method,
 		write_val = write (file_handle->fd, buffer, num_bytes);
 	while (write_val == -1
 	       && errno == EINTR
-	       && ! gnome_vfs_context_check_cancellation(context));
+	       && ! gnome_vfs_context_check_cancellation (context));
 
 	if (write_val == -1) {
 		*bytes_written = 0;
@@ -317,7 +342,7 @@ do_write (GnomeVFSMethod *method,
 	}
 }
 
-
+
 static gint
 seek_position_to_unix (GnomeVFSSeekPosition position)
 {
@@ -349,7 +374,7 @@ do_seek (GnomeVFSMethod *method,
 
 	if (LSEEK (file_handle->fd, offset, lseek_whence) == -1) {
 		if (errno == ESPIPE)
-			return GNOME_VFS_ERROR_NOTSUPPORTED;
+			return GNOME_VFS_ERROR_NOT_SUPPORTED;
 		else
 			return gnome_vfs_result_from_errno ();
 	}
@@ -370,7 +395,7 @@ do_tell (GnomeVFSMethod *method,
 	offset = LSEEK (file_handle->fd, 0, SEEK_CUR);
 	if (offset == -1) {
 		if (errno == ESPIPE)
-			return GNOME_VFS_ERROR_NOTSUPPORTED;
+			return GNOME_VFS_ERROR_NOT_SUPPORTED;
 		else
 			return gnome_vfs_result_from_errno ();
 	}
@@ -398,9 +423,9 @@ do_truncate_handle (GnomeVFSMethod *method,
 		switch (errno) {
 		case EBADF:
 		case EROFS:
-			return GNOME_VFS_ERROR_READONLY;
+			return GNOME_VFS_ERROR_READ_ONLY;
 		case EINVAL:
-			return GNOME_VFS_ERROR_NOTSUPPORTED;
+			return GNOME_VFS_ERROR_NOT_SUPPORTED;
 		default:
 			return GNOME_VFS_ERROR_GENERIC;
 		}
@@ -413,22 +438,29 @@ do_truncate (GnomeVFSMethod *method,
 	     GnomeVFSFileSize where,
 	     GnomeVFSContext *context)
 {
-	if (truncate (uri->text, where) == 0) {
+	gchar *path;
+
+	path = get_path_from_uri (uri);
+	if (path == NULL)
+		return GNOME_VFS_ERROR_INVALID_URI;
+
+	if (truncate (path, where) == 0) {
+		g_free (path);
 		return GNOME_VFS_OK;
 	} else {
+		g_free (path);
 		switch (errno) {
 		case EBADF:
 		case EROFS:
-			return GNOME_VFS_ERROR_READONLY;
+			return GNOME_VFS_ERROR_READ_ONLY;
 		case EINVAL:
-			return GNOME_VFS_ERROR_NOTSUPPORTED;
+			return GNOME_VFS_ERROR_NOT_SUPPORTED;
 		default:
 			return GNOME_VFS_ERROR_GENERIC;
 		}
 	}
 }
 
-
 struct _DirectoryHandle
 {
 	GnomeVFSURI *uri;
@@ -436,7 +468,7 @@ struct _DirectoryHandle
 	GnomeVFSFileInfoOptions options;
 	const GList *meta_keys;
 
-	struct dirent current_entry;
+	struct dirent *current_entry;
 
 	gchar *name_buffer;
 	gchar *name_ptr;
@@ -452,31 +484,37 @@ directory_handle_new (GnomeVFSURI *uri,
 		      const GList *meta_keys,
 		      const GnomeVFSDirectoryFilter *filter)
 {
-	DirectoryHandle *new;
+	DirectoryHandle *result;
 	gchar *full_name;
 	guint full_name_len;
 
-	new = g_new (DirectoryHandle, 1);
+	result = g_new (DirectoryHandle, 1);
 
-	new->uri = gnome_vfs_uri_ref (uri);
-	new->dir = dir;
+	result->uri = gnome_vfs_uri_ref (uri);
+	result->dir = dir;
 
-	MAKE_ABSOLUTE (full_name, uri->text);
+	/* Reserve extra space for readdir_r, see man page */
+	result->current_entry = g_malloc (sizeof (struct dirent) + GET_PATH_MAX() + 1);
+
+	full_name = get_path_from_uri (uri);
+	g_assert (full_name != NULL); /* already done by caller */
 	full_name_len = strlen (full_name);
 
-	new->name_buffer = g_malloc (full_name_len + GET_PATH_MAX () + 2);
-	memcpy (new->name_buffer, full_name, full_name_len);
+	result->name_buffer = g_malloc (full_name_len + GET_PATH_MAX () + 2);
+	memcpy (result->name_buffer, full_name, full_name_len);
 	
 	if (full_name_len > 0 && full_name[full_name_len - 1] != '/')
-		new->name_buffer[full_name_len++] = '/';
+		result->name_buffer[full_name_len++] = '/';
 
-	new->name_ptr = new->name_buffer + full_name_len;
+	result->name_ptr = result->name_buffer + full_name_len;
 
-	new->options = options;
-	new->meta_keys = meta_keys;
-	new->filter = filter;
+	g_free (full_name);
 
-	return new;
+	result->options = options;
+	result->meta_keys = meta_keys;
+	result->filter = filter;
+
+	return result;
 }
 
 static void
@@ -484,51 +522,36 @@ directory_handle_destroy (DirectoryHandle *directory_handle)
 {
 	gnome_vfs_uri_unref (directory_handle->uri);
 	g_free (directory_handle->name_buffer);
+	g_free (directory_handle->current_entry);
 	g_free (directory_handle);
 }
 
-
 /* MIME detection code.  */
-
-/* Hm, this is a bit messy.  */
-
 static void
-set_mime_type (GnomeVFSFileInfo *info,
-	       const gchar *full_name,
+get_mime_type (GnomeVFSFileInfo *info,
+	       const char *full_name,
 	       GnomeVFSFileInfoOptions options,
-	       struct stat *statbuf)
+	       struct stat *stat_buffer)
 {
-	const gchar *mime_type;
+	const char *mime_type;
 
-	if (options & GNOME_VFS_FILE_INFO_FASTMIMETYPE) {
-		const gchar *mime_name;
-
-		if ((options & GNOME_VFS_FILE_INFO_FOLLOWLINKS)
-		    && info->type != GNOME_VFS_FILE_TYPE_BROKENSYMLINK
-		    && info->symlink_name != NULL)
-			mime_name = info->symlink_name;
-		else
-			mime_name = full_name;
-
-		mime_type = gnome_mime_type_or_default (mime_name, NULL);
-
-		if (mime_type == NULL)
-			mime_type = gnome_vfs_mime_type_from_mode (statbuf->st_mode);
+	mime_type = NULL;
+	if ((options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) == 0
+		&& (info->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK)) {
+		/* we are a symlink and aren't asked to follow -
+		 * return the type for a symlink
+		 */
+		mime_type = "x-special/symlink";
 	} else {
-		/* FIXME: This will also stat the file for us...  Which is
-                   not good at all, as we already have the stat info when we
-                   get here, but there is no other way to do this with the
-                   current gnome-libs.  */
-		/* FIXME: We actually *always* follow symlinks here.  It
-                   needs fixing.  */
-		mime_type = gnome_mime_type_from_magic (full_name);
+		mime_type = gnome_vfs_get_file_mime_type (full_name,
+			stat_buffer, (options & GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE) != 0);
 	}
 
+	g_assert (mime_type);
 	info->mime_type = g_strdup (mime_type);
 	info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
 }
 
-
 static gchar *
 read_link (const gchar *full_name)
 {
@@ -539,9 +562,12 @@ read_link (const gchar *full_name)
 	buffer = g_malloc (size);
           
 	while (1) {
-		guint read_size;
+		int read_size;
 
                 read_size = readlink (full_name, buffer, size);
+		if (read_size < 0) {
+			return NULL;
+		}
                 if (read_size < size) {
 			buffer[read_size] = 0;
 			return buffer;
@@ -559,24 +585,30 @@ get_stat_info (GnomeVFSFileInfo *file_info,
 {
 	struct stat statbuf;
 
-	if (statptr == NULL)
+	if (statptr == NULL) {
 		statptr = &statbuf;
+	}
 
-	if (lstat (full_name, statptr) != 0)
-		return gnome_vfs_result_from_errno ();
-
-	if (!S_ISLNK (statptr->st_mode)) {
-		GNOME_VFS_FILE_INFO_SET_SYMLINK (file_info, FALSE);
-	} else {
-		GNOME_VFS_FILE_INFO_SET_SYMLINK (file_info, TRUE);
-		file_info->symlink_name = read_link (full_name);
-		file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME;
-
-		if (options & GNOME_VFS_FILE_INFO_FOLLOWLINKS) {
-			if (stat (full_name, statptr) != 0)
-				file_info->type
-					= GNOME_VFS_FILE_TYPE_BROKENSYMLINK;
+	if (options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) {
+		if (stat (full_name, statptr) != 0) {
+			/* failed to resolve the link or some other problem */
+			return gnome_vfs_result_from_errno ();
 		}
+	} else if (lstat (full_name, statptr) != 0) {
+		return gnome_vfs_result_from_errno ();
+	}
+
+	if (S_ISLNK (statptr->st_mode)) {
+		/* we are dealing with a symlink and the follow flag is off */
+
+		g_assert ((options & GNOME_VFS_FILE_INFO_FOLLOW_LINKS) == 0);
+		
+		file_info->type = GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK;
+		file_info->symlink_name = read_link (full_name);
+		if (file_info->symlink_name == NULL) {
+			return gnome_vfs_result_from_errno ();
+		}
+		file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_SYMLINK_NAME;
 	}
 
 	gnome_vfs_stat_to_file_info (file_info, statptr);
@@ -618,9 +650,12 @@ do_open_directory (GnomeVFSMethod *method,
 	gchar *directory_name;
 	DIR *dir;
 
-	MAKE_ABSOLUTE (directory_name, uri->text);
+	directory_name = get_path_from_uri (uri);
+	if (directory_name == NULL)
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	dir = opendir (directory_name);
+	g_free (directory_name);
 	if (dir == NULL)
 		return gnome_vfs_result_from_errno ();
 
@@ -662,7 +697,7 @@ read_directory (DirectoryHandle *handle,
 	gchar *full_name;
 	gboolean filter_called;
 
-	/* This makes sure we do try to filter the file more than
+	/* This makes sure we don't try to filter the file more than
            once.  */
 	filter_called = FALSE;
 
@@ -670,28 +705,27 @@ read_directory (DirectoryHandle *handle,
 	if (filter != NULL) {
 		filter_needs = gnome_vfs_directory_filter_get_needs (filter);
 	} else {
-		/* Shut up stupid compiler.  */
 		filter_needs = GNOME_VFS_DIRECTORY_FILTER_NEEDS_NOTHING;
 	}
 
-	if (readdir_r (handle->dir,
-		       &handle->current_entry,
-		       &result) != 0)
+	if (readdir_r (handle->dir, handle->current_entry, &result) != 0) {
 		return gnome_vfs_result_from_errno ();
-
-	if (result == NULL)
+	}
+	
+	if (result == NULL) {
 		return GNOME_VFS_ERROR_EOF;
+	}
 
 	info->name = g_strdup (result->d_name);
 
 	if (filter != NULL
-	    && ! filter_called
-	    && ! (filter_needs
+	    && !filter_called
+	    && (filter_needs
 		  & (GNOME_VFS_DIRECTORY_FILTER_NEEDS_TYPE
 		     | GNOME_VFS_DIRECTORY_FILTER_NEEDS_STAT
 		     | GNOME_VFS_DIRECTORY_FILTER_NEEDS_MIMETYPE
-		     | GNOME_VFS_DIRECTORY_FILTER_NEEDS_METADATA))) {
-		if (! gnome_vfs_directory_filter_apply (filter, info)) {
+		     | GNOME_VFS_DIRECTORY_FILTER_NEEDS_METADATA)) == 0){
+		if (!gnome_vfs_directory_filter_apply (filter, info)) {
 			*skip = TRUE;
 			return GNOME_VFS_OK;
 		}
@@ -702,16 +736,15 @@ read_directory (DirectoryHandle *handle,
 	strcpy (handle->name_ptr, result->d_name);
 	full_name = handle->name_buffer;
 
-	/* FIXME: Correct?  */
-	if (get_stat_info (info, full_name, handle->options, &statbuf)
-	    != GNOME_VFS_OK)
+	/* FIXME bugzilla.eazel.com 1223: Correct?  */
+	if (get_stat_info (info, full_name, handle->options, &statbuf) != GNOME_VFS_OK)
 		return GNOME_VFS_ERROR_INTERNAL;
 
 	if (filter != NULL
-	    && ! filter_called
-	    && ! (filter_needs
+	    && !filter_called
+	    && (filter_needs
 		  & (GNOME_VFS_DIRECTORY_FILTER_NEEDS_MIMETYPE
-		     | GNOME_VFS_DIRECTORY_FILTER_NEEDS_METADATA))) {
+		     | GNOME_VFS_DIRECTORY_FILTER_NEEDS_METADATA)) == 0) {
 		if (! gnome_vfs_directory_filter_apply (filter, info)) {
 			*skip = TRUE;
 			return GNOME_VFS_OK;
@@ -719,13 +752,14 @@ read_directory (DirectoryHandle *handle,
 		filter_called = TRUE;
 	}
 
-	if (handle->options & GNOME_VFS_FILE_INFO_GETMIMETYPE)
-		set_mime_type (info, full_name, handle->options, &statbuf);
+	if (handle->options & GNOME_VFS_FILE_INFO_GET_MIME_TYPE) {
+		get_mime_type (info, full_name, handle->options, &statbuf);
+	}
 
 	if (filter != NULL
-	    && ! filter_called
-	    && ! (filter_needs & GNOME_VFS_DIRECTORY_FILTER_NEEDS_METADATA)) {
-		if (! gnome_vfs_directory_filter_apply (filter, info)) {
+	    && !filter_called
+	    && (filter_needs & GNOME_VFS_DIRECTORY_FILTER_NEEDS_METADATA) == 0) {
+		if (!gnome_vfs_directory_filter_apply (filter, info)) {
 			*skip = TRUE;
 			return GNOME_VFS_OK;
 		}
@@ -734,8 +768,8 @@ read_directory (DirectoryHandle *handle,
 
 	gnome_vfs_set_meta_for_list (info, full_name, handle->meta_keys);
 
-	if (filter != NULL && ! filter_called) {
-		if (! gnome_vfs_directory_filter_apply (filter, info)) {
+	if (filter != NULL && !filter_called) {
+		if (!gnome_vfs_directory_filter_apply (filter, info)) {
 			*skip = TRUE;
 			return GNOME_VFS_OK;
 		}
@@ -768,7 +802,7 @@ do_read_directory (GnomeVFSMethod *method,
 	return result;
 }
 
-
+
 static GnomeVFSResult
 do_get_file_info (GnomeVFSMethod *method,
 		  GnomeVFSURI *uri,
@@ -781,21 +815,29 @@ do_get_file_info (GnomeVFSMethod *method,
 	gchar *full_name;
 	struct stat statbuf;
 
-	MAKE_ABSOLUTE (full_name, uri->text);
+	full_name = get_path_from_uri (uri);
+	if (full_name == NULL)
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	file_info->valid_fields = GNOME_VFS_FILE_INFO_FIELDS_NONE;
 
-	file_info->name = gnome_vfs_uri_extract_short_name (uri);
+	file_info->name = get_base_from_uri (uri);
+	g_assert (file_info->name != NULL);
 
-	result = get_stat_info (file_info, full_name, options, NULL);
-	if (result != GNOME_VFS_OK)
+	result = get_stat_info (file_info, full_name, options, &statbuf);
+	if (result != GNOME_VFS_OK) {
+		g_free (full_name);
 		return result;
+	}
 
-	if (options & GNOME_VFS_FILE_INFO_GETMIMETYPE)
-		set_mime_type (file_info, full_name, options, &statbuf);
+	if (options & GNOME_VFS_FILE_INFO_GET_MIME_TYPE) {
+		get_mime_type (file_info, full_name, options, &statbuf);
+	}
 
 	gnome_vfs_set_meta_for_list (file_info, full_name, meta_keys);
 
+	g_free (full_name);
+
 	return GNOME_VFS_OK;
 }
 
@@ -814,26 +856,33 @@ do_get_file_info_from_handle (GnomeVFSMethod *method,
 
 	file_handle = (FileHandle *) method_handle;
 
-	MAKE_ABSOLUTE (full_name, file_handle->uri->text);
+	full_name = get_path_from_uri (file_handle->uri);
+	if (full_name == NULL)
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	file_info->valid_fields = GNOME_VFS_FILE_INFO_FIELDS_NONE;
 
-	file_info->name = gnome_vfs_uri_extract_short_name (file_handle->uri);
+	file_info->name = get_base_from_uri (file_handle->uri);
+	g_assert (file_info->name != NULL);
 
 	result = get_stat_info_from_handle (file_info, file_handle,
 					    options, &statbuf);
-	if (result != GNOME_VFS_OK)
+	if (result != GNOME_VFS_OK) {
+		g_free (full_name);
 		return result;
+	}
 
-	if (options & GNOME_VFS_FILE_INFO_GETMIMETYPE)
-		set_mime_type (file_info, full_name, options, &statbuf);
+	if (options & GNOME_VFS_FILE_INFO_GET_MIME_TYPE)
+		get_mime_type (file_info, full_name, options, &statbuf);
 
 	gnome_vfs_set_meta_for_list (file_info, full_name, meta_keys);
 
+	g_free (full_name);
+
 	return GNOME_VFS_OK;
 }
 
-
+
 static gboolean
 do_is_local (GnomeVFSMethod *method,
 	     const GnomeVFSURI *uri)
@@ -844,7 +893,7 @@ do_is_local (GnomeVFSMethod *method,
 	return TRUE;
 }
 
-
+
 static GnomeVFSResult
 do_make_directory (GnomeVFSMethod *method,
 		   GnomeVFSURI *uri,
@@ -854,13 +903,19 @@ do_make_directory (GnomeVFSMethod *method,
 	gint retval;
 	gchar *full_name;
 
-	MAKE_ABSOLUTE (full_name, uri->text);
+	full_name = get_path_from_uri (uri);
+	if (full_name == NULL)
+		return GNOME_VFS_ERROR_INVALID_URI;
+
 	retval = mkdir (full_name, perm);
 
-	if (retval == 0)
-		return GNOME_VFS_OK;
-	else
+	g_free (full_name);
+
+	if (retval != 0) {
 		return gnome_vfs_result_from_errno ();
+	}
+
+	return GNOME_VFS_OK;
 }
 
 static GnomeVFSResult
@@ -871,13 +926,99 @@ do_remove_directory (GnomeVFSMethod *method,
 	gchar *full_name;
 	gint retval;
 
-	MAKE_ABSOLUTE (full_name, uri->text);
+	full_name = get_path_from_uri (uri);
+	if (full_name == NULL)
+		return GNOME_VFS_ERROR_INVALID_URI;
+
 	retval = rmdir (full_name);
 
-	if (retval == 0)
-		return GNOME_VFS_OK;
-	else
+	g_free (full_name);
+
+	if (retval != 0) {
 		return gnome_vfs_result_from_errno ();
+	}
+
+	return GNOME_VFS_OK;
+}
+
+static GnomeVFSResult
+do_find_directory (GnomeVFSMethod *method,
+		   GnomeVFSURI *near_uri,
+		   GnomeVFSFindDirectoryKind kind,
+		   GnomeVFSURI **result_uri,
+		   gboolean create_if_needed,
+		   guint permissions,
+		   GnomeVFSContext *context)
+{
+	gint retval;
+	char *full_name_near;
+	struct stat near_item_stat;
+	struct stat home_volume_stat;
+	const char *home_directory;
+	char *target_directory;
+
+	target_directory = NULL;
+	*result_uri = NULL;
+
+	full_name_near = get_path_from_uri (near_uri);
+	if (full_name_near == NULL)
+		return GNOME_VFS_ERROR_INVALID_URI;
+
+	home_directory = g_get_home_dir();
+
+	if (gnome_vfs_context_check_cancellation (context)) {
+		g_free (full_name_near);
+		return GNOME_VFS_ERROR_CANCELLED;
+	}
+
+	retval = stat (full_name_near, &near_item_stat);
+	g_free (full_name_near);
+	if (retval != 0)
+		return gnome_vfs_result_from_errno ();
+
+	if (gnome_vfs_context_check_cancellation (context))
+		return GNOME_VFS_ERROR_CANCELLED;
+
+	retval = stat (home_directory, &home_volume_stat);
+	if (retval != 0)
+		return gnome_vfs_result_from_errno ();
+
+	if (gnome_vfs_context_check_cancellation (context))
+		return GNOME_VFS_ERROR_CANCELLED;
+
+	if (near_item_stat.st_dev != home_volume_stat.st_dev)
+		/* FIXME bugzilla.eazel.com 1187 */
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+
+	switch (kind) {
+	case GNOME_VFS_DIRECTORY_KIND_TRASH:
+		target_directory = g_strconcat(home_directory, G_DIR_SEPARATOR_S, 
+			"Trash", NULL);
+		break;
+		
+	case GNOME_VFS_DIRECTORY_KIND_DESKTOP:
+		target_directory = g_strconcat(home_directory, G_DIR_SEPARATOR_S, 
+			"Desktop", NULL);
+		break;
+
+	default:
+		break;
+	}
+
+	if (target_directory == NULL)
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
+
+	if (create_if_needed && !g_file_exists (target_directory))
+		mkdir (target_directory, permissions);
+
+	if (!g_file_exists (target_directory)) {
+		g_free (target_directory);
+		return GNOME_VFS_ERROR_NOT_FOUND;
+	}
+	*result_uri = gnome_vfs_uri_new (target_directory);
+	g_free (target_directory);
+
+	return GNOME_VFS_OK;
 }
 
 static GnomeVFSResult
@@ -895,49 +1036,52 @@ rename_helper (const gchar *old_full_name,
 		/* If we are not allowed to replace an existing file, return an
                    error.  */
 		if (! force_replace)
-			return GNOME_VFS_ERROR_FILEEXISTS;
+			return GNOME_VFS_ERROR_FILE_EXISTS;
 		old_exists = TRUE;
 	} else {
 		old_exists = FALSE;
 	}
 
-	if (gnome_vfs_context_check_cancellation(context))
+	if (gnome_vfs_context_check_cancellation (context))
 		return GNOME_VFS_ERROR_CANCELLED;
 
 	retval = rename (old_full_name, new_full_name);
 
-	/* FIXME?  The following assumes that, if `new_uri' and `old_uri' are
-           on different file systems, `rename()' will always return `EXDEV'
-           instead of `EISDIR', even if the old file is not a directory while
-           the new one is.  If this is not the case, we have to stat() both
-	   the old and new file.  */
-
+	/* FIXME bugzilla.eazel.com 1186: The following assumes that,
+	 * if `new_uri' and `old_uri' are on different file systems,
+	 * `rename()' will always return `EXDEV' instead of `EISDIR',
+	 * even if the old file is not a directory while the new one
+	 * is. If this is not the case, we have to stat() both the
+	 * old and new file.
+	 */
 	if (retval != 0 && errno == EISDIR && force_replace && old_exists) {
 		/* The Unix version of `rename()' fails if the original file is
 		   not a directory, while the new one is.  But we have been
 		   explicitly asked to replace the destination name, so if the
 		   new name points to a directory, we remove it manually.  */
 		if (S_ISDIR (statbuf.st_mode)) {
-			if (gnome_vfs_context_check_cancellation(context))
+			if (gnome_vfs_context_check_cancellation (context))
 				return GNOME_VFS_ERROR_CANCELLED;
 			retval = rmdir (new_full_name);
 			if (retval != 0) {
-				/* FIXME: Maybe we could be more accurate
-				   here?  */
-				return GNOME_VFS_ERROR_DIRECTORYNOTEMPTY;
+				/* FIXME bugzilla.eazel.com 1185:
+				 * Maybe we could be more accurate here?
+				 */
+				return GNOME_VFS_ERROR_DIRECTORY_NOT_EMPTY;
 			}
 
-			if (gnome_vfs_context_check_cancellation(context))
+			if (gnome_vfs_context_check_cancellation (context))
 				return GNOME_VFS_ERROR_CANCELLED;
 
 			retval = rename (old_full_name, new_full_name);
 		}
 	}
 
-	if (retval == 0)
-		return GNOME_VFS_OK;
-	else
+	if (retval != 0) {
 		return gnome_vfs_result_from_errno ();
+	}
+
+	return GNOME_VFS_OK;
 }
 
 static GnomeVFSResult
@@ -949,12 +1093,25 @@ do_move (GnomeVFSMethod *method,
 {
 	gchar *old_full_name;
 	gchar *new_full_name;
+	GnomeVFSResult result;
 
-	MAKE_ABSOLUTE (new_full_name, new_uri->text);
-	MAKE_ABSOLUTE (old_full_name, old_uri->text);
+	old_full_name = get_path_from_uri (old_uri);
+	if (old_full_name == NULL)
+		return GNOME_VFS_ERROR_INVALID_URI;
 
-	return rename_helper (old_full_name, new_full_name, force_replace,
-			      context);
+	new_full_name = get_path_from_uri (new_uri);
+	if (new_full_name == NULL) {
+		g_free (old_full_name);
+		return GNOME_VFS_ERROR_INVALID_URI;
+	}
+
+	result = rename_helper (old_full_name, new_full_name,
+				force_replace, context);
+
+	g_free (old_full_name);
+	g_free (new_full_name);
+
+	return result;
 }
 
 static GnomeVFSResult
@@ -965,12 +1122,74 @@ do_unlink (GnomeVFSMethod *method,
 	gchar *full_name;
 	gint retval;
 
-	MAKE_ABSOLUTE (full_name, uri->text);
+	full_name = get_path_from_uri (uri);
+	if (full_name == NULL) {
+		return GNOME_VFS_ERROR_INVALID_URI;
+	}
+
 	retval = unlink (full_name);
-	if (retval == 0)
-		return GNOME_VFS_OK;
-	else
+
+	g_free (full_name);
+
+	if (retval != 0) {
 		return gnome_vfs_result_from_errno ();
+	}
+
+	return GNOME_VFS_OK;
+}
+
+static GnomeVFSResult
+do_create_symbolic_link (GnomeVFSMethod *method,
+			 GnomeVFSURI *uri,
+			 const char *target_reference,
+			 GnomeVFSContext *context)
+{
+	const char *link_scheme, *target_scheme;
+	char *link_full_name, *target_full_name;
+	GnomeVFSResult result;
+	GnomeVFSURI *target_uri;
+
+	g_assert (target_reference != NULL);
+	g_assert (uri != NULL);
+	
+	/* what we actually want is a function that takes a gchar* and tells whether it
+	   is a valid URI...does such a beast exist? */
+	target_uri = gnome_vfs_uri_new (target_reference);
+	g_assert (target_uri != NULL);
+
+	link_scheme = gnome_vfs_uri_get_scheme (uri);
+	g_assert (link_scheme != NULL);
+
+	target_scheme = gnome_vfs_uri_get_scheme (target_uri);
+	if (target_scheme == NULL) {
+		target_scheme = "file";
+	}
+	
+	if ((strcmp (link_scheme, "file") == 0) && (strcmp (target_scheme, "file") == 0)) {
+		/* symlink between two places on the local filesystem */
+		if (strncmp (target_reference, "file", 4) != 0)
+			/* target_reference wasn't a full URI */
+			target_full_name = strdup (target_reference); 
+		else 
+			target_full_name = get_path_from_uri (target_uri);
+		link_full_name = get_path_from_uri (uri);
+
+		if (symlink (target_full_name, link_full_name) != 0) {
+			result = gnome_vfs_result_from_errno ();
+		} else {
+			result = GNOME_VFS_OK;
+		}
+
+		g_free (target_full_name);
+		g_free (link_full_name);
+	} else {
+		/* FIXME: do a URI link */
+		result = GNOME_VFS_ERROR_NOT_SUPPORTED;
+	}
+
+	gnome_vfs_uri_unref (target_uri);
+
+	return result;
 }
 
 static GnomeVFSResult
@@ -984,17 +1203,19 @@ do_check_same_fs (GnomeVFSMethod *method,
 	struct stat sa, sb;
 	gint retval;
 
-	MAKE_ABSOLUTE (full_name_a, a->text);
-	MAKE_ABSOLUTE (full_name_b, b->text);
-
+	full_name_a = get_path_from_uri (a);
 	retval = stat (full_name_a, &sa);
+	g_free (full_name_a);
+
 	if (retval != 0)
 		return gnome_vfs_result_from_errno ();
 
-	if (gnome_vfs_context_check_cancellation(context))
+	if (gnome_vfs_context_check_cancellation (context))
 		return GNOME_VFS_ERROR_CANCELLED;
 
+	full_name_b = get_path_from_uri (b);
 	retval = stat (full_name_b, &sb);
+	g_free (full_name_b);
 
 	if (retval != 0)
 		return gnome_vfs_result_from_errno ();
@@ -1013,45 +1234,61 @@ do_set_file_info (GnomeVFSMethod *method,
 {
 	gchar *full_name;
 
-	MAKE_ABSOLUTE (full_name, uri->text);
+	full_name = get_path_from_uri (uri);
+	if (full_name == NULL)
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	if (mask & GNOME_VFS_SET_FILE_INFO_NAME) {
 		GnomeVFSResult result;
-		gchar *dir;
-		gchar *new, *old;
+		gchar *dir, *encoded_dir;
+		gchar *new_name;
 
-		old = full_name;
+		encoded_dir = gnome_vfs_uri_extract_dirname (uri);
+		dir = gnome_vfs_unescape_string (encoded_dir, "/");
+		g_free (encoded_dir);
+		g_assert (dir != NULL);
 
-		dir = gnome_vfs_uri_extract_dirname (uri);
-		new = g_concat_dir_and_file (dir, info->name);
+		new_name = g_concat_dir_and_file (dir, info->name);
 
-		result = rename_helper (old, new, FALSE, context);
+		result = rename_helper (full_name, new_name, FALSE, context);
 
 		g_free (dir);
-		g_free (new);
+		g_free (new_name);
 
-		if (result != GNOME_VFS_OK)
+		if (result != GNOME_VFS_OK) {
+			g_free (full_name);
 			return result;
+		}
 	}
 
-	if (gnome_vfs_context_check_cancellation(context))
+	if (gnome_vfs_context_check_cancellation (context)) {
+		g_free (full_name);
 		return GNOME_VFS_ERROR_CANCELLED;
+	}
 
 	if (mask & GNOME_VFS_SET_FILE_INFO_PERMISSIONS) {
-		if (chmod (full_name, info->permissions) != 0)
+		if (chmod (full_name, info->permissions) != 0) {
+			g_free (full_name);
 			return gnome_vfs_result_from_errno ();
+		}
 	}
 
-	if (gnome_vfs_context_check_cancellation(context))
+	if (gnome_vfs_context_check_cancellation (context)) {
+		g_free (full_name);
 		return GNOME_VFS_ERROR_CANCELLED;
+	}
 
 	if (mask & GNOME_VFS_SET_FILE_INFO_OWNER) {
-		if (chown (full_name, info->uid, info->gid) != 0)
+		if (chown (full_name, info->uid, info->gid) != 0) {
+			g_free (full_name);
 			return gnome_vfs_result_from_errno ();
+		}
 	}
 
-	if (gnome_vfs_context_check_cancellation(context))
+	if (gnome_vfs_context_check_cancellation (context)) {
+		g_free (full_name);
 		return GNOME_VFS_ERROR_CANCELLED;
+	}
 
 	if (mask & GNOME_VFS_SET_FILE_INFO_TIME) {
 		struct utimbuf utimbuf;
@@ -1059,10 +1296,14 @@ do_set_file_info (GnomeVFSMethod *method,
 		utimbuf.actime = info->atime;
 		utimbuf.modtime = info->mtime;
 
-		if (utime (full_name, &utimbuf) != 0)
+		if (utime (full_name, &utimbuf) != 0) {
+			g_free (full_name);
 			return gnome_vfs_result_from_errno ();
+		}
 	}
 
+	g_free (full_name);
+
 	return GNOME_VFS_OK;
 }
 
@@ -1087,7 +1328,9 @@ static GnomeVFSMethod method = {
 	do_unlink,
 	do_check_same_fs,
 	do_set_file_info,
-	do_truncate
+	do_truncate,
+	do_find_directory,
+	do_create_symbolic_link
 };
 
 GnomeVFSMethod *
