@@ -111,7 +111,7 @@ static GnomeVFSMethod method = {
 	do_write,		/* write */
 	NULL,			/* seek */
 	NULL,			/* tell */
-	NULL,			/* truncate_handle FIXME */
+	NULL,			/* truncate_handle FIXME bugzilla.eazel.com 1175 */
 	NULL,			/* open_directory */
 	NULL,			/* close_directory */
 	NULL,			/* read_directory */
@@ -122,7 +122,9 @@ static GnomeVFSMethod method = {
 	NULL,			/* remove_directory */
 	NULL,			/* rename */
 	NULL,                   /* unlink */
-	NULL /* truncate */
+	NULL, 			/* truncate */
+	NULL,			/* find_directory */
+	NULL                    /* create_symbolic_link */
 };
 
 #define RETURN_IF_FAIL(action)			\
@@ -206,7 +208,7 @@ gzip_method_handle_init_for_deflate (GZipMethodHandle *handle)
 	handle->zstream.next_out = handle->buffer;
 	handle->zstream.avail_out = Z_BUFSIZE;
 
-	/* FIXME: We want this to be user-configurable.  */
+	/* FIXME bugzilla.eazel.com 1174: We want this to be user-configurable.  */
 	if (deflateInit2 (&handle->zstream, Z_DEFAULT_COMPRESSION,
 			  Z_DEFLATED, -MAX_WBITS, MAX_MEM_LEVEL,
 			  Z_DEFAULT_STRATEGY) != Z_OK) {
@@ -226,10 +228,10 @@ result_from_z_result (gint z_result)
 {
 	switch (z_result) {
 	case Z_OK:
-	case Z_STREAM_END:          /* FIXME? */
+	case Z_STREAM_END: /* FIXME bugzilla.eazel.com 1173: Is this right? */
 		return GNOME_VFS_OK;
 	case Z_DATA_ERROR:
-		return GNOME_VFS_ERROR_CORRUPTEDDATA;
+		return GNOME_VFS_ERROR_CORRUPTED_DATA;
 	default:
 		return GNOME_VFS_ERROR_INTERNAL;
 	}
@@ -250,7 +252,7 @@ skip_string (GnomeVFSHandle *handle)
 		RETURN_IF_FAIL (result);
 
 		if (bytes_read != 1)
-			return GNOME_VFS_ERROR_WRONGFORMAT;
+			return GNOME_VFS_ERROR_WRONG_FORMAT;
 	} while (c != 0);
 
 	return GNOME_VFS_OK;
@@ -270,7 +272,7 @@ skip (GnomeVFSHandle *handle,
 	RETURN_IF_FAIL (result);
 
 	if (bytes_read != num_bytes)
-		return GNOME_VFS_ERROR_WRONGFORMAT;
+		return GNOME_VFS_ERROR_WRONG_FORMAT;
 
 	return TRUE;
 }
@@ -306,7 +308,7 @@ read_guint32 (GnomeVFSHandle *handle,
 	RETURN_IF_FAIL (result);
 
 	if (bytes_read != 4)
-		return GNOME_VFS_ERROR_WRONGFORMAT;
+		return GNOME_VFS_ERROR_WRONG_FORMAT;
 
 	*value_return = (buffer[0] | (buffer[1] << 8)
 			 | (buffer[2] << 16) | (buffer[3] << 24));
@@ -332,19 +334,19 @@ read_gzip_header (GnomeVFSHandle *handle,
 	RETURN_IF_FAIL (result);
 
 	if (bytes_read != GZIP_HEADER_SIZE)
-		return GNOME_VFS_ERROR_WRONGFORMAT;
+		return GNOME_VFS_ERROR_WRONG_FORMAT;
 
 	if (buffer[0] != GZIP_MAGIC_1 || buffer[1] != GZIP_MAGIC_2)
-		return GNOME_VFS_ERROR_WRONGFORMAT;
+		return GNOME_VFS_ERROR_WRONG_FORMAT;
 
 	mode = buffer[2];
 	if (mode != 8) /* Mode: deflate */
-		return GNOME_VFS_ERROR_WRONGFORMAT;
+		return GNOME_VFS_ERROR_WRONG_FORMAT;
 
 	flags = buffer[3];
 
 	if (flags & GZIP_FLAG_RESERVED)
-		return GNOME_VFS_ERROR_WRONGFORMAT;
+		return GNOME_VFS_ERROR_WRONG_FORMAT;
 
 	if (flags & GZIP_FLAG_EXTRA_FIELD) {
 		guchar tmp[2];
@@ -352,9 +354,9 @@ read_gzip_header (GnomeVFSHandle *handle,
 
 		if (gnome_vfs_read (handle, tmp, 2, &bytes_read)
 		    || bytes_read != 2)
-			return GNOME_VFS_ERROR_WRONGFORMAT;
+			return GNOME_VFS_ERROR_WRONG_FORMAT;
 		if (! skip (handle, tmp[0] | (tmp[0] << 8)))
-			return GNOME_VFS_ERROR_WRONGFORMAT;
+			return GNOME_VFS_ERROR_WRONG_FORMAT;
 	}
 
 	if (flags & GZIP_FLAG_ORIG_NAME)
@@ -477,12 +479,12 @@ do_open (GnomeVFSMethod *method,
 
 	/* We don't allow any paths in the GZIP file.  */
 	if (uri->text != NULL && uri->text[0] != 0)
-		return GNOME_VFS_ERROR_INVALIDURI;
+		return GNOME_VFS_ERROR_INVALID_URI;
 
 	parent_uri = uri->parent;
 
 	if (open_mode & GNOME_VFS_OPEN_RANDOM)
-		return GNOME_VFS_ERROR_NOTSUPPORTED;
+		return GNOME_VFS_ERROR_NOT_SUPPORTED;
 
 	result = gnome_vfs_open_uri (&parent_handle, parent_uri, open_mode);
 	RETURN_IF_FAIL (result);
@@ -508,7 +510,7 @@ do_open (GnomeVFSMethod *method,
 		result = write_gzip_header (parent_handle);
 		RETURN_IF_FAIL (result);
 
-				/* FIXME modification_time */
+		/* FIXME bugzilla.eazel.com 1172: need to set modification_time */
 		gzip_handle = gzip_method_handle_new (parent_handle,
 						      (time_t) 0,
 						      uri,
@@ -541,7 +543,7 @@ do_create (GnomeVFSMethod *method,
 	_GNOME_VFS_METHOD_PARAM_CHECK (method_handle != NULL);
 	_GNOME_VFS_METHOD_PARAM_CHECK (uri != NULL);
 
-	return GNOME_VFS_ERROR_NOTSUPPORTED; /* FIXME */
+	return GNOME_VFS_ERROR_NOT_SUPPORTED; /* FIXME bugzilla.eazel.com 1170 */
 }
 
 
@@ -567,7 +569,7 @@ do_close (GnomeVFSMethod *method,
 	if (result == GNOME_VFS_OK)
 		result = gnome_vfs_close (gzip_handle->parent_handle);
 
-	/* FIXME: How do we deal with close errors?  */
+	/* FIXME bugzilla.eazel.com 1169: How do we deal with close errors?  */
 	gzip_method_handle_destroy (gzip_handle);
 
 	return result;
@@ -605,7 +607,7 @@ fill_buffer (GZipMethodHandle *gzip_handle,
 	return GNOME_VFS_OK;
 }
 
-/* TODO:
+/* FIXME bugzilla.eazel.com 1165: TODO:
    - Concatenated GZIP file handling.  */
 static GnomeVFSResult
 do_read (GnomeVFSMethod *method,
@@ -621,7 +623,6 @@ do_read (GnomeVFSMethod *method,
 	int z_result;
 	guchar *crc_start;
 
-	/* FIXME: Wrong.  */
 	*bytes_read = 0;
 
 	crc_start = buffer;
@@ -642,7 +643,6 @@ do_read (GnomeVFSMethod *method,
 	zstream->next_out = buffer;
 	zstream->avail_out = num_bytes;
 
-	/* FIXME: Clean up.  */
 	while (zstream->avail_out != 0) {
 		result = fill_buffer (gzip_handle, num_bytes);
 		RETURN_IF_FAIL (result);
@@ -661,12 +661,12 @@ do_read (GnomeVFSMethod *method,
 			result = read_guint32 (gzip_handle->parent_handle,
 					       &crc);
 			if (result != GNOME_VFS_OK || crc != gzip_handle->crc) {
-				/* FIXME?  Set VFS error instead?  */
+				/* FIXME bugzilla.eazel.com 1167: Set VFS error instead?  */
 				gzip_handle->last_z_result = Z_DATA_ERROR;
 				break;
 			}
 		} else if (z_result != Z_OK) {	
-			/* FIXME: Concatenated GZIP files?  */
+			/* FIXME bugzilla.eazel.com 1165: Concatenated GZIP files?  */
 			gzip_handle->last_z_result = z_result;
 		}
 
