@@ -22,13 +22,11 @@
 
 #include "gicon.h"
 
-#include "gnome-vfs.h"
+#include "libgnomevfs/gnome-vfs.h"
 
 #include "gnome-file-selection.h"
 #include "gnome-file-selection-history.h"
 
-#include "listing-iconic.xpm"
-
 
 /* GnomeIconList parameters.  */
 #define ICON_LIST_SPACING          64
@@ -166,8 +164,7 @@ populate_callback (GnomeVFSAsyncHandle *handle,
 	if (list != NULL) {
 		info = gnome_vfs_directory_list_current (list);
 		while (info != NULL) {
-			if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY
-			    && ! GNOME_VFS_FILE_INFO_SYMLINK (info)) {
+			if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
 				add_directory_to_clist (fs->directory_clist,
 							info->name);
 			} else {
@@ -212,9 +209,9 @@ start_populating (GnomeFileSelection *fs)
 
 	gnome_vfs_async_load_directory (&fs->priv->async_handle,
 					fs->directory,
-					(GNOME_VFS_FILE_INFO_GETMIMETYPE
-					 | GNOME_VFS_FILE_INFO_FASTMIMETYPE
-					 | GNOME_VFS_FILE_INFO_FOLLOWLINKS),
+					(GNOME_VFS_FILE_INFO_GET_MIME_TYPE
+					 | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE
+					 | GNOME_VFS_FILE_INFO_FOLLOW_LINKS),
 					NULL,
 					NULL,
 					FALSE,
@@ -261,7 +258,6 @@ update_directory_combo_list (GnomeFileSelection *fs)
 	gtk_signal_handler_block (GTK_OBJECT (combo_list),
 				  fs->priv->combo_selection_changed_id);
 
-	/* FIXME: Wasted `strdup ()'.  */
 	popdown_list = g_list_prepend (popdown_list, g_strdup (G_DIR_SEPARATOR_S));
 
 	p1 = fs->directory + 1;
@@ -283,7 +279,6 @@ update_directory_combo_list (GnomeFileSelection *fs)
 		popdown_list = g_list_prepend (popdown_list, s);
 	}
 
-	/* FIXME: Wasted `strdup ()'.  */
 	if (strcmp (fs->directory, G_DIR_SEPARATOR_S) != 0)
 		popdown_list = g_list_prepend (popdown_list,
 					      g_strconcat (fs->directory,
@@ -648,8 +643,9 @@ setup_directory_combo_and_toolbar (GnomeFileSelection *fs)
 
 	fs->directory_combo = gtk_combo_new ();
 
-	/* FIXME: I cannot get editing to work nicely, so I just disable
-	   it for now.  */
+	/* FIXME bugzilla.eazel.com 1123: I cannot get editing to work
+	 * nicely, so I just disable it for now.
+	 */
 	gtk_editable_set_editable (GTK_EDITABLE
 				  (GTK_COMBO (fs->directory_combo)->entry),
 				  FALSE);
@@ -928,7 +924,8 @@ destroy (GtkObject *object)
 	while (fs->priv->populating_in_progress) ;
 	clean_file_list (fs->priv);
 
-	/* FIXME free the handle. */
+	if (fs->priv->async_handle != NULL)
+		gnome_vfs_async_cancel (fs->priv->async_handle);
 
 	g_free (fs->priv);
 
@@ -961,7 +958,7 @@ class_init (GnomeFileSelectionClass *class)
 
 	widget_class->map = map;
 
-	gicon_init ();		/* FIXME */
+	gicon_init ();		/* FIXME bugzilla.eazel.com 1121 */
 }
 
 static void
@@ -999,7 +996,7 @@ init (GnomeFileSelection *fs)
 		{
 			fs->paned = gtk_hpaned_new ();
 
-			/* FIXME: This does not appear to work!  */
+			/* FIXME bugzilla.eazel.com 1126: This does not appear to work!  */
 			gtk_paned_set_position (GTK_PANED (fs->paned), 100);
 
 			gtk_box_pack_start (GTK_BOX (dialog->vbox), fs->paned, TRUE, TRUE, 0);
@@ -1026,7 +1023,7 @@ init (GnomeFileSelection *fs)
 
 	/* Notice that this must be done *before* setting the list type,
 	   because the latter will cause the widget to be populated.
-	   FIXME: This could be done via `change_dir ()'.  */
+	   FIXME bugzilla.eazel.com 1125: This could be done via `change_dir ()'.  */
 	fs->directory = g_get_current_dir ();
 	gnome_file_selection_history_add (fs->priv->history, fs->directory);
 	update_directory_combo_entry (fs);
