@@ -90,15 +90,15 @@ type_to_string (GnomeVFSFileType type)
 		return "Regular";
 	case GNOME_VFS_FILE_TYPE_DIRECTORY:
 		return "Directory";
-	case GNOME_VFS_FILE_TYPE_BROKENSYMLINK:
-		return "Broken symlink";
+	case GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK:
+		return "Symbolic Link";
 	case GNOME_VFS_FILE_TYPE_FIFO:
 		return "FIFO";
 	case GNOME_VFS_FILE_TYPE_SOCKET:
 		return "Socket";
-	case GNOME_VFS_FILE_TYPE_CHARDEVICE:
+	case GNOME_VFS_FILE_TYPE_CHARACTER_DEVICE:
 		return "Character device";
-	case GNOME_VFS_FILE_TYPE_BLOCKDEVICE:
+	case GNOME_VFS_FILE_TYPE_BLOCK_DEVICE:
 		return "Block device";
 	default:
 		return "???";
@@ -113,9 +113,10 @@ print_list (GnomeVFSDirectoryList *list, guint num_entries)
 
 	info = gnome_vfs_directory_list_current (list);
 	for (i = 0; i < num_entries && info != NULL; i++) {
-		printf ("  File `%s'%s (%s, %s), size %Ld, mode %04o\n",
+		printf ("  File `%s'%s (%s, %s), "
+			"size %"GNOME_VFS_SIZE_FORMAT_STR", mode %04o\n",
 			info->name,
-			(info->flags & GNOME_VFS_FILE_FLAGS_SYMLINK) ? " [link]" : "",
+			(info->type == GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK) ? " [link]" : "",
 			type_to_string (info->type),
 			gnome_vfs_file_info_get_mime_type (info),
 			info->size, info->permissions);
@@ -157,9 +158,8 @@ int
 main (int argc, char **argv)
 {
 	GnomeVFSAsyncHandle *handle;
-	GnomeVFSResult result;
 	poptContext popt_context;
-	char **args;
+	const char **args;
 	gchar *text_uri;
 	GnomeVFSDirectorySortRule sort_rules[] = {
 		GNOME_VFS_DIRECTORY_SORT_BYNAME,
@@ -209,12 +209,12 @@ main (int argc, char **argv)
 	}
 
 	num_entries = 0;
-	result = gnome_vfs_async_load_directory
+	gnome_vfs_async_load_directory
 		(&handle, /* handle */
 		 text_uri, /* text_uri */
-		 (GNOME_VFS_FILE_INFO_GETMIMETYPE
-		  | GNOME_VFS_FILE_INFO_FASTMIMETYPE
-		  | GNOME_VFS_FILE_INFO_FOLLOWLINKS), /* options */
+		 (GNOME_VFS_FILE_INFO_GET_MIME_TYPE
+		  | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE
+		  | GNOME_VFS_FILE_INFO_FOLLOW_LINKS), /* options */
 		 NULL, /* meta_keys */
 		 sort ? sort_rules : NULL, /* sort_rules */
 		 FALSE, /* reverse_order */
@@ -249,6 +249,8 @@ main (int argc, char **argv)
 
 	puts ("All done");
 
+	gnome_vfs_shutdown ();
+
 	while (1)
 		;
 
